#!/bin/sh
#  -*- Mode: Tcl -*-
# anatempura.tcl. \
exec tclsh "$0" ${1+"$@"}

package require Tk
package require tile

# anatempura --
#
# This file implements package Anatempura, a graphical front end and 
# monitor to Tempura
#
# Copyright (C) 1998-2019 Antonio Cau

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id: anatempura.tcl 292 2017-12-19 12:14:14Z cau $

# Namespace tile::theme::strl --
#
#   ADD COMMENTS HERE
#

namespace eval tile::theme::strl {

    package provide tile::theme::strl 0.9

    
    variable colors
    array set colors {
	-frame  	"#eadaa5"
	-lighter	"#f5f5f5"
	-window	 	"#ffffff"
	-selectbg	"#ffff33"
	-selectfg	"#000000"
	-disabledfg	"#666666"
    }

    ttk::style theme create strl -parent clam -settings {

	ttk::style configure . \
	    -borderwidth 	1 \
	    -background 	$colors(-frame) \
	    -fieldbackground	$colors(-window) \
	    -troughcolor	$colors(-lighter) \
	    -selectbackground	$colors(-selectbg) \
	    -selectforeground	$colors(-selectfg) \
	    ;
	ttk::style map . -foreground [list disabled $colors(-disabledfg)]

	## Buttons.
	#
	ttk::style configure TButton \
	    -anchor center -width -11 -padding "1 1" -relief raised -shiftrelief 1 \
	    -highlightthickness 1 -highlightcolor $colors(-frame) -foreground black

	ttk::style map TButton -relief {
	    {pressed !disabled} 	sunken
	    {active !disabled} 	raised
	} -highlightcolor {alternate black}

        ttk::style configure Red.TButton \
	    -anchor center -width -11 -padding "1 1" -relief raised -shiftrelief 1 \
	    -highlightthickness 1 -highlightcolor $colors(-frame) -foreground red


        ttk::style configure TMenubutton \
	    -anchor center -width -11 -padding "1 1" -relief raised -shiftrelief 1 \
	    -highlightthickness 1 -highlightcolor $colors(-frame)

	ttk::style map TMenubutton -relief {
	    {pressed !disabled} 	sunken
	    {active !disabled} 	raised
	} -highlightcolor {alternate black}
        
	ttk::style configure TRadiobutton -padding 1
	ttk::style configure TCheckbutton -padding 1

	## Toolbar buttons.
	#
        ttk::style configure RedToolbutton \
	    -width 0 -relief flat -borderwidth 2 -padding 3 \
	    -background $colors(-frame) -foreground red ;
        ttk::style map RedToolbutton -background [list active $colors(-selectbg)] 
	ttk::style map RedToolbutton -foreground [list active $colors(-selectfg)] 
	ttk::style map RedToolbutton -relief {
	    disabled 	flat
	    selected	sunken  
	    pressed 	sunken  
	    active  	raised
	}

        
	ttk::style configure Toolbutton \
	    -width 0 -relief flat -borderwidth 2 -padding 3 \
	    -background $colors(-frame) -foreground #000000 ;
	ttk::style map Toolbutton -background [list active $colors(-selectbg)] 
	ttk::style map Toolbutton -foreground [list active $colors(-selectfg)] 
	ttk::style map Toolbutton -relief {
	    disabled 	flat
	    selected	sunken  
	    pressed 	sunken  
	    active  	raised
	}
        
	## Entry widgets.
	#
	ttk::style configure TEntry \
	    -selectborderwidth 1 -padding 2 -insertwidth 2 -font TkTextFont
	ttk::style configure TCombobox \
	    -selectborderwidth 1 -padding 2 -insertwidth 2 -font TkTextFont

	## Notebooks.
	#
        ttk::style configure TNotebook -padding 0 -tabmargins {2 2 1 0}
        ttk::style configure TNotebook.Tab -padding {4 2}
        ttk::style map TNotebook.Tab -expand [list selected {2 2 1 0}]

	## Labelframes.
	#
	ttk::style configure TLabelframe -borderwidth 2 -relief groove

	## Scrollbars.
	#
        ttk::style configure TScrollbar \
	    -width 12 -arrowsize 12 
	
 
	ttk::style layout Vertical.TScrollbar {
	    Scrollbar.trough -children {
		Scrollbar.uparrow -side top
		Scrollbar.downarrow -side bottom
		Scrollbar.uparrow -side bottom
		Vertical.Scrollbar.thumb -side top -expand true -sticky ns
	    }
	}

	ttk::style layout Horizontal.TScrollbar {
	    Scrollbar.trough -children {
		Scrollbar.leftarrow -side left
		Scrollbar.rightarrow -side right
		Scrollbar.leftarrow -side right
		Horizontal.Scrollbar.thumb -side left -expand true -sticky we
	    }
	}
    }
}


# Namespace history --
# from http://wiki.tcl.tk/1109
# plus new addto
# adapted for use with text widget
#
namespace eval history {
    proc add? {w} {
        variable $w
        variable n$w
        upvar 0 $w hist
        #set s [set ::[$w cget -textvariable]]
        set s [$w get limit end-1c]
        if {$s == ""} return
        if [string compare $s [lindex $hist end]] {
            lappend hist $s
            set n$w [llength $hist]
        }
    }
    proc addto {w s} {
        variable $w
        variable n$w
        upvar 0 $w hist
        if {$s == ""} return
        if [string compare $s [lindex $hist end]] {
            lappend hist $s
            set n$w [llength $hist]
        }
    }
    proc move {w where} {
        variable $w
        variable n$w
        upvar 0 $w hist
        incr n$w $where
        if {[set n$w]<0} {set n$w 0}
        if {[set n$w]>=[llength $hist]+1} {
            set n$w [llength $hist]
        }
        #set ::[$w cget -textvar] [lindex $hist [set n$w]]
        $w delete limit end-1c
        $w mark set insert end
        $w mark set limit insert
        $w insert insert [lindex $hist [set n$w]]
        $w see end
    }
    proc for {type name args} {
        switch -- $type {
            text {
                uplevel $type $name $args
                bind $name <Up> {history::move %W -1; break}
                bind $name <Down> {history::move %W 1}
                bind $name <Next> {history::move %W 99999}
               # bind $name <Return> {history::add? %W}
                variable $name {}
                variable n$name 0
            }
            default {error "usage: history::for text <w> <args>"}
        }
    }
 }


package provide anatempura 1.00
# Namespace anatempura --
#
#   ADD COMMENTS HERE
#
namespace eval anatempura {;
 global tempura_io proggy_iol reset at_input at_prompt show_output veri syntax_error

# init --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc init {} {
    variable TEMPURA
    global tempura_io env

    if {$TEMPURA(tempura_dir) == "" || ![file isdirectory $TEMPURA(tempura_dir)]} {
        set TEMPURA(tempura_dir)  [pwd]
    }
    set tmp $TEMPURA(tempura_dir)
    set TEMPURA(tempura_dir) ""
    if {[file exists "$env(HOME)/$TEMPURA(rc_file)"]} {
        source "$env(HOME)/$TEMPURA(rc_file)"
    }
    if {[file exists "./$TEMPURA(rc_file)"]} {
        source "./$TEMPURA(rc_file)"
    }
    if {$TEMPURA(tempura_dir) != "" && [file isdirectory $TEMPURA(tempura_dir)]} {
        newdir $TEMPURA(tempura_dir)
    } else {
        set TEMPURA(tempura_dir) $tmp
    }
    ttk::setTheme strl
    option add *.font $TEMPURA(HelvBig) widgetDefault
    ttk::style configure . -font $TEMPURA(HelvBig)
}






# openfile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc openfile {} {
    variable TEMPURA
    global tempura_io
    
    set types {
        { {Tempura File}       {.t}     }
        { {Tcl/Tk File}        {.tcl}   }
        { {C File}             {.c}     }
        { {C\# File}           {.cs}    }
        { {Java File}          {.java}  }
        { {Scala File}         {.scala} }
        { {Verilog File}       {.v}     }
    }
    set tmp $TEMPURA(base_file)
    set TEMPURA(base_file) \
        [tk_getOpenFile -defaultextension ".t" -filetypes $types -initialdir $TEMPURA(tempura_dir) ]
    catch {grab .top18} err
    grab release .top18
    if {$TEMPURA(base_file) != ""} {
      newdir [file dir $TEMPURA(base_file)]
      setpath
      set TEMPURA(tempura_file) [file tail $TEMPURA(base_file)]
      buildeditmenu [file join $TEMPURA(tempura_dir) $TEMPURA(tempura_file)]
      set ttt [file rootname $TEMPURA(tempura_file)]
      loadfile $ttt  
      set TEMPURA(file_lbl) "$ttt"
      wm title .top18 "AnaTempura:  $TEMPURA(file_lbl)"
    } else {
      set TEMPURA(base_file) $tmp
    }
}

proc setpath {} {
    variable TEMPURA
    global tempura_io  

    cd $TEMPURA(tempura_dir)
    tempura_send "set path =\"$TEMPURA(tempura_dir)\"."
}


# reload --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc reload {} {
    variable TEMPURA
    global tempura_io

    if { $TEMPURA(tempura_file) != "" && $TEMPURA(tempura_dir)!=""} {
        set lfile [file rootname $TEMPURA(tempura_file)]
        loadfile $lfile
        buildeditmenu [file join $TEMPURA(tempura_dir) $TEMPURA(tempura_file)]
    }
}

# rerun --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc rerun {} {
    variable TEMPURA
    global tempura_io 
    
    if { $TEMPURA(rerun) != "" } {
        if {[string range $TEMPURA(rerun) end end] == "." } {
            set temp [string trimright $TEMPURA(rerun) "."]
        } else {
            set temp $TEMPURA(rerun)
        }
        if [catch {$TEMPURA(runmenu) index $temp} index] {
            tempura_send "$TEMPURA(rerun)"
        } else {
            $TEMPURA(runmenu) invoke $index
        }
    }
}



# reset --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc reset {} {
    variable TEMPURA
    global tempura_io reset proggy_iol


    foreach id [array names proggy_iol] {
        stopexternal $proggy_iol($id)
    }
    
    set tmp [pwd]
    set reset 1
    cd $TEMPURA(tempura_path)
    tempura_send "exit."
    chan close $tempura_io
    set tempura_io [tempura_spawn]
    set reset 0
    set x 0
    after 500 {set x 1}
    vwait x
    setpath
}


# break --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc break {} {
    variable TEMPURA
    global tempura_io at_input at_prompt proggy_iol external

    foreach id [array names proggy_iol] {
        $external($id).pop invoke 1
    }

    if [catch {exec kill -s SIGINT [pid $tempura_io]} err] {
        tkterm::o_insert "Oh no, I can't send an interrupt: $err.\n" sta
    } else {
        if { $at_input || $at_prompt } {
            tempura_send "."
        }
    }
    update idletasks
}



# savefile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc savefile {} {
    variable TEMPURA

    set types {
    {{Output  Files}      {.out} }
    }  
    set TEMPURA(save_file) \
        [tk_getSaveFile -defaultextension ".out" -filetypes $types -initialdir $TEMPURA(tempura_dir)    ]
    catch {grab .top18} err
    grab release .top18
    if {$TEMPURA(save_file) != ""} {
        newdir [file dir $TEMPURA(save_file)]
        set TEMPURA(out_file) [file tail $TEMPURA(save_file)]
        cd $TEMPURA(tempura_dir)
        set output [open $TEMPURA(out_file) w]
        puts $output [$TEMPURA(output) get 0.0 end]
        close $output
    } 

}

# newfile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc newfile {} {
    variable TEMPURA 

    set types {
        { {Tempura File}       {.t}     }
        { {Tcl/Tk File}        {.tcl}   }
        { {C File}             {.c}     }
        { {C\# File}           {.cs}    }
        { {Java File}          {.java}  }
        { {Scala File}         {.scala} }
        { {Verilog File}       {.v}     }
    }
    set tmp $TEMPURA(base_file)
    set TEMPURA(base_file) \
     [tk_getSaveFile -defaultextension ".t" -filetypes $types -initialdir $TEMPURA(tempura_dir) ]
    catch {grab .top18} err
    grab release .top18
    if {$TEMPURA(base_file) != "" && ![file exists $TEMPURA(base_file)]} {
        set fId [open $TEMPURA(base_file) w]
        close $fId
        newdir [file dir $TEMPURA(base_file)]
        setpath
        set TEMPURA(tempura_file) [file tail $TEMPURA(base_file)]
        buildeditmenu [file join $TEMPURA(tempura_dir) $TEMPURA(tempura_file)]
        set ttt [file rootname $TEMPURA(tempura_file)]
        set TEMPURA(file_lbl) "$ttt"
        wm title .top18 "AnaTempura:  $TEMPURA(file_lbl)"
        editfile $TEMPURA(tempura_file)
    } else {
        set TEMPURA(base_file) $tmp
    }
}

# setdir --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc setdir {} {
    variable TEMPURA 
    global tempura_io 

    set tmp $TEMPURA(tempura_dir)
    set TEMPURA(tempura_dir) \
        [tk_chooseDirectory -initialdir $TEMPURA(tempura_dir) -mustexist true]
    catch {grab .top18} err
    grab release .top18
    if {$TEMPURA(tempura_dir) != "" && [file isdirectory $TEMPURA(tempura_dir)]} {
        newdir $TEMPURA(tempura_dir) 
        setpath 
    } else {
        set TEMPURA(tempura_dir) $tmp
    }
}

# newdir --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#    d          directory
# Results:
#   Returns ...
#
proc newdir {d} {
    variable TEMPURA
    if {![file isdirectory $d]} {return}
    set TEMPURA(tempura_dir) "[convertfilename $d]"
}

# convertfilename --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#    f          filename
# Results:
#   Returns ...
#
proc convertfilename {f} {
    return $f
}

# separator --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#    
# Results:
#   Returns ...
#
proc separator {} {
    switch -- $::tcl_platform(platform) {
        unix    {return ":"}
        windows {return ";"}
        default {error "unknown platform"}
    }
}

# findinclasspath --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#    filename   filename
#    cp         classpath
# Results:
#   Returns ...
#
proc findinclasspath {filename cp id} {
    set containjar 0
    foreach path [split $cp [separator]] {
        if {[file extension $path] == ".jar"} {
            if {[file exists $path]} {
                tkterm::e_insert $id "Oh no, I will not check for $filename in $path.\n" sta
                set containjar 1
            } else {
                tkterm::e_insert $id "Oh no, I can not find $path.\n" sta
                if {$containjar == 1} {
                    set containjar 1
                } else {
                    set containjar 0
                }
            }
        } else {
            set rootfilename [file rootname $filename]
            set ext [file extension $filename]
            set dotfilelist [join [split $rootfilename .] "/"]
            set try [file join $path $dotfilelist]$ext
            if [file exists $try] {return $try}
        }
    }
    if {$containjar == 0} {
        return -1
    } else {
        return 0
    }
} 


# update_title
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#    runid      current process id
# Results:
#   Returns ...
#
proc update_title {runid} {
    global external runnames externalwindow

    if {![info exists external($runid)]} {
        visual::new_tab $runid
    } else {
        set tabid [expr $runid +3]
        .top18.cpd22.n tab $tabid -text "External $runnames($runid)"
    }
    if {[info exists externalwindow($runid)]} {
        if {[winfo exists .top3$runid]} {
            wm title .top3$runid "External Output $runnames($runid)"
        }
    }
}
    
# buildeditmenu --
#
#   ADD COMMENTS HERE
#
# buildeditmenu:  Given a base file, find all the files loaded by
# it, and put them in the edit menu.
#
# Arguments:
#   args	comments
#    f           filename
# Results:
#   Returns ...
#
proc buildeditmenu {f} {
    variable TEMPURA
    global veri external runlist runnames

    $TEMPURA(editmenu) delete 0 last
    $TEMPURA(checkmenu) delete 0 last
    $TEMPURA(runmenu) delete 0 last

    set dir [file dir $f]
    set main [file tail $f]
    cd $dir 
    $TEMPURA(editmenu) add command -label "$main" \
         -font $TEMPURA(HelvBig) \
         -command "anatempura::editfile {$f}"   
    $TEMPURA(checkmenu) add command -label "$main" \
         -font $TEMPURA(HelvBig)\
         -command "anatempura::loadfile [file rootname $main]"
  
    set fId [open $f r]
    set proggy "0"
    set jproggy "0"
    set iproggy "0"
    set wave ""
 
    set flist {}
    set dlist {}
    
    set runlist(plist) {}
    set runlist(analist) {}
    set runlist(vlist) {}
    set runlist(rmilist) {}
    set runlist(jlist) {}
    #set runlist(jcplist) {}
    set runlist(rsjlist) {}
    #set runlist(rsjcplist) {}
    set runlist(rcjlist) {}
    #set runlist(rcjcplist) {}
    set runlist(jjlist) {}
    set runlist(cslist) {}
    set runlist(sclist) {}
    #set runlist(sccplist) {}
    set runlist(sbtlist) {}

    foreach pr [array names runlist] {
        lappend runlist($pr) "1"
    }

    set runid 0
    
    while {[gets $fId line] >= 0} {
        if {[string first "load" $line] != -1 } {
            lappend flist "[lindex [split $line "\"\""] 1]"
        } elseif {[string first "/* run */" $line] != -1 } {
            set linetmp [string trimleft $line]
            set linetmp [string trimleft $linetmp  "/* run */"]
            set linetmp [string trimleft $linetmp]
            set linetmp [string trimleft $linetmp  "define"]
            set linetmp [string trimleft $linetmp]
            set linetmp [lindex [split $linetmp "="] 0]
            set linetmp [string trimright $linetmp]
            lappend dlist  $linetmp
        } elseif {[string first "/* prog " $line] != -1 } {
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            #set proggy "[lindex [split [string trimleft $line " "] " "] 2]"
            #set proggy "[lindex $jtemp 2]"
            set proggy "[lrange  $jtemp 2 [expr $ltemp -3]]"
            #set pnum   "[lindex [split [string trimleft $line " "] " "] 3]"
            #set pnum "[lindex $jtemp 3]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(plist) "$runid $proggy"
            set tproggy [lindex $proggy 0]
            if {[file exists [file join $TEMPURA(tempura_dir) $tproggy.c]]} {
                $TEMPURA(checkmenu) add command -label $tproggy.c \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::checkprog {$tproggy} $runid"
                $TEMPURA(editmenu) add command -label $tproggy.c \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$tproggy.c}"
            }
            incr runid
        } elseif {[string first "/* anatempura " $line] != -1 } {
            set jtemp [regexp -all -inline {\S+} $line]
            set proggy "dummy"
            #set pnum   "[lindex [split [string trimleft $line " "] " "] 2]"
            set pnum "[lindex $jtemp 2]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(analist) "$runid $proggy"
            incr runid
        } elseif {[string first "/* veri " $line] != -1 } {
            set jtemp [regexp -all -inline {\S+} $line]
            #set proggy "[lindex [split [string trimleft $line " "] " "] 2]"
            set proggy "[lindex $jtemp 2]"
            #set pnum   "[lindex [split [string trimleft $line " "] " "] 3]"
            set pnum "[lindex $jtemp 3]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(vlist) "$runid $proggy"
            $TEMPURA(checkmenu) add command -label $proggy \
                -font $TEMPURA(HelvBig)\
                -command "anatempura::checkveri {$proggy} $runid"
            $TEMPURA(editmenu) add command -label $proggy \
                -font $TEMPURA(HelvBig)\
                -command "anatempura::editfile {$proggy}"
            incr runid
        } elseif {[string first "/* rmiregistry " $line] != -1 } {
            set proggy "dummy"
            set jtemp [regexp -all -inline {\S+} $line]
            #set pnum   "[lindex [split [string trimleft $line " "] " "] 2]"
            set pnum "[lindex $jtemp 2]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(rmilist) "$runid $proggy"
            incr runid
        } elseif {[string first "/* java " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set proggy "[lrange  $jtemp 2 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(jlist) "$runid . $proggy"
            #lappend runlist(jcplist) "."
            set cp "."
            set jproggy [findinclasspath $proggy.java $cp $runid]
            if {$jproggy != -1 && $jproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::checkjava {$jproggy} {$cp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$jproggy}"
            }
            incr runid
        } elseif {[string first "/* javacp " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set proggy "[lrange  $jtemp 3 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            set cp "[lindex $jtemp 2]"
            lappend runlist(jlist) "$runid $cp $proggy"
            
            #lappend runlist(jcplist) "[join [lrange $jtemp 2 [expr $ltemp -4]]]"
            #set cp "[join [lrange $jtemp 2 [expr $ltemp -4]]]"
            #lappend runlist(jcplist) "[lindex $jtemp 2]"
            #set cp "[lindex $jtemp 2]"
            set jproggy [findinclasspath $proggy.java $cp $runid]
            if {$jproggy != -1 && $jproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::checkjava {$jproggy} {$cp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$jproggy}"
            }
            incr runid
        } elseif {[string first "/* rmiserver " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set iproggy "[lindex  $jtemp 2]"
            set proggy "[lrange  $jtemp 3 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(rsjlist) "$runid . $proggy"
            #lappend runlist(rsjcplist) "."
            set rscp "."
            set rsjproggy [findinclasspath $proggy.java $rscp $runid]
            set irsjproggy [findinclasspath $iproggy.java $rscp $runid]
            if {$rsjproggy != -1 && $rsjproggy != 0 && $irsjproggy != -1 && $irsjproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::rmicheckjava {$irsjproggy} {$rsjproggy} {$rscp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$rsjproggy}"
                $TEMPURA(editmenu) add command -label $iproggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$irsjproggy}"
            }
            incr runid
        } elseif {[string first "/* rmiservercp " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set iproggy "[lindex  $jtemp 3]"
            set proggy "[lrange  $jtemp 4 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            set rscp "[lindex $jtemp 2]"
            lappend runlist(rsjlist) "$runid $rscp $proggy"
            #lappend runlist(rsjcplist) "[join [lrange $jtemp 2 [expr $ltemp -5]]]"
            #set rscp "[join [lrange $jtemp 2 [expr $ltemp -5]]]"
            set rsjproggy [findinclasspath $proggy.java $rscp $runid]
            set irsjproggy [findinclasspath $iproggy.java $rscp $runid]
            if {$rsjproggy != -1 && $rsjproggy != 0 && $irsjproggy != -1 && $irsjproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::rmicheckjava {$irsjproggy} {$rsjproggy} {$rscp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$rsjproggy}"
                $TEMPURA(editmenu) add command -label $iproggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$irsjproggy}"
            }
            incr runid
        } elseif {[string first "/* rmiclient " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set iproggy "[lindex  $jtemp 2]"
            set proggy "[lrange  $jtemp 3 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(rcjlist) "$runid . $proggy"
            #lappend runlist(rcjcplist) "."
            set rccp "."
            set rcjproggy [findinclasspath $proggy.java $rccp $runid]
            set ircjproggy [findinclasspath $iproggy.java $rccp $runid]
            if {$rcjproggy != -1 && $rcjproggy != 0 && $ircjproggy != -1 && $ircjproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::rmicheckjava {$ircjproggy} {$rcjproggy} {$rccp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$rcjproggy}"
                $TEMPURA(editmenu) add command -label $iproggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$ircjproggy}"
            }
            incr runid
        } elseif {[string first "/* rmiclientcp " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set iproggy "[lindex  $jtemp 3]"
            set proggy "[lrange  $jtemp 4 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            set rccp "[lindex $jtemp 2]"
            lappend runlist(rcjlist) "$runid $rccp $proggy"
            set rcjproggy [findinclasspath $proggy.java $rccp $runid]
            set ircjproggy [findinclasspath $iproggy.java $rccp $runid]
            if {$rcjproggy != -1 && $rcjproggy != 0 && $ircjproggy != -1 && $ircjproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::rmicheckjava {$ircjproggy} {$rcjproggy} {$rccp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$rcjproggy}"
                $TEMPURA(editmenu) add command -label $iproggy.java \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$ircjproggy}"
            }
            incr runid
        } elseif {[string first "/* scala " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set proggy "[lrange  $jtemp 2 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(sclist) "$runid . $proggy"
            #lappend runlist(sccplist) "."
            set sccp "."
            set scproggy [findinclasspath $proggy.scala $sccp $runid]
            if {$scproggy != -1 && $scproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.scala \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::checkscala {$scproggy} {$sccp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.scala \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$scproggy}"
            }
            incr runid
       } elseif {[string first "/* scalacp " $line] != -1 } {
            #set jtemp [split [string trimleft $line " "] " "]
            set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set proggy "[lrange  $jtemp 3 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            set sccp "[lindex $jtemp 2]"
            lappend runlist(sclist) "$runid $sccp $proggy"
            
            #lappend runlist(sccplist) "[join [lrange $jtemp 2 [expr $ltemp -4]]]"
            #set sccp "[join [lrange $jtemp 2 [expr $ltemp -4]]]"
            #lappend runlist(sccplist) "[lindex $jtemp 2]"
            
            set scproggy [findinclasspath $proggy.scala $sccp $runid]
            if {$scproggy != -1 && $scproggy != 0} {
                $TEMPURA(checkmenu) add command -label $proggy.scala \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::checkscala {$scproggy} {$sccp} $runid"
                $TEMPURA(editmenu) add command -label $proggy.scala \
                    -font $TEMPURA(HelvBig)\
                    -command "anatempura::editfile {$scproggy}"
            }
            incr runid
       } elseif {[string first "/* sbt " $line] != -1 } {
           #set jtemp [split [string trimleft $line " "] " "]
           set jtemp [regexp -all -inline {\S+} $line]
            set ltemp [llength $jtemp]
            set proggy "[lrange  $jtemp 2 [expr $ltemp -3]]"
            set pnum   "[lindex  $jtemp [expr $ltemp -2]]"
            set runnames($runid) "$pnum"
            update_title $runid
            lappend runlist(sbtlist) "$runid $proggy"
            incr runid
       } elseif {[string first "/* javajar " $line] != -1 } {
           set jtemp [regexp -all -inline {\S+} $line]
           set ltemp [llength $jtemp]
           #set proggy "[lindex [split [string trimleft $line " "] " "] 2]"
           set proggy "[lrange $jtemp 2 [expr $ltemp -3]]" 
           #set pnum   "[lindex [split [string trimleft $line " "] " "] 3]"
           set pnum "[lindex $jtemp [expr $ltemp -2]]"
           set runnames($runid) "$pnum"
           update_title $runid
           lappend runlist(jjlist) "$runid $proggy"
           incr runid
       } elseif {[string first "/* csc " $line] != -1 } {
           set jtemp [regexp -all -inline {\S+} $line]
           set ltemp [llength $jtemp]
           #set proggy "[lindex [split [string trimleft $line " "] " "] 2]"
           set proggy "[lrange $jtemp 2 [expr $ltemp -3]]"
           #set pnum   "[lindex [split [string trimleft $line " "] " "] 3]"
           set pnum "[lindex $jtemp [expr $ltemp -2]]"
           set runnames($runid) "$pnum"
           update_title $runid
           lappend runlist(cslist) "$runid $proggy" 
           if {[file exists [file join $TEMPURA(tempura_dir) $proggy.cs]]} {
               $TEMPURA(checkmenu) add command -label $proggy.cs \
                   -font $anatempura::TEMPURA(HelvBig)\
                   -command "anatempura::checkcsc {$proggy} $runid"
               $TEMPURA(editmenu) add command -label $proggy.cs \
                   -font $TEMPURA(HelvBig)\
                   -command "anatempura::editfile {$proggy.cs}"
           }
           incr runid
       } elseif {[string first "/* vdmp " $line] != -1 } {
            set jtemp [regexp -all -inline {\S+} $line]
           #set wave "[lindex [split [string trimleft $line " "] " "] 2]"
           set wave "[lindex $jtemp 2]"
       } elseif {[string first "/* tcl " $line] != -1 } {
           visual::Window show .top23
           set jtemp [regexp -all -inline {\S+} $line]
           #set TEMPURA(file_tcl) "[lindex [split [string trimleft $line " "] " "] 2].tcl"
           set TEMPURA(file_tcl) "[lindex $jtemp 2].tcl"
            $TEMPURA(editmenu) add command -label $TEMPURA(file_tcl) \
                -font $TEMPURA(HelvBig)\
                -command "anatempura::editfile {$TEMPURA(file_tcl)}"
       }
    }
    close $fId
  
    foreach f $flist {
        if {[file extension $f] == ""} {set f "$f.t"}
        if {![file exists [file join $dir $f]]} continue
        set fId [open $f r]
        while {[gets $fId line] >= 0} {
            if {[string first "/* run */" $line] != -1 } {
                set linetmp [string trimleft $line]
                set linetmp [string trimleft $linetmp  "/* run */"]
                set linetmp [string trimleft $linetmp]
                set linetmp [string trimleft $linetmp  "define"]
                set linetmp [string trimleft $linetmp]
                set linetmp [lindex [split $linetmp "="] 0]
                set linetmp [string trimright $linetmp]
                lappend dlist  $linetmp
            }
        }
        close $fId
        $TEMPURA(editmenu) add command -label $f \
            -font $TEMPURA(HelvBig)\
            -command "anatempura::editfile {$f}"
        $TEMPURA(checkmenu) add command -label $f \
            -font $TEMPURA(HelvBig)\
            -command "anatempura::loadfile [file rootname $f]"
    }
    set num 0
    if {$wave != ""} {
        $TEMPURA(runmenu) add command -label "wave $wave" \
            -font $TEMPURA(HelvBig)\
            -command "anatempura::wavefile $wave "
        incr num   
    }
  
    foreach f $dlist {
        if {$num < 24} {
            $TEMPURA(runmenu) add command -label "run $f" \
                -font $TEMPURA(HelvBig)\
                -command "anatempura::runfile {$f} "
        } else { 
            set num 0
            $TEMPURA(runmenu) add command -label "run $f" \
                -font $TEMPURA(HelvBig) -columnbreak 1\
                -command "anatempura::runfile {$f} "
        }
        incr num
    }
  
}

# wavefile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc wavefile {f} {
    variable TEMPURA

    if {[file exists [file join $TEMPURA(tempura_dir) $f]]} {
        if {[file exists $TEMPURA(wave)]} {
            exec  [file nativename $TEMPURA(wave)] [file nativename [file join $TEMPURA(tempura_dir) $f]] &
        }   
    }
}

# editfile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#     f           filename
# Results:
#   Returns ...
#
proc editfile {f} {
    variable TEMPURA

    if {[file exists [file join $TEMPURA(tempura_dir) $f]] } {
        if {$TEMPURA(tempura_editor)=="vi"} {
            exec xterm -e vi [file nativename [file join $TEMPURA(tempura_dir) $f]] &
        } else {
            exec $TEMPURA(tempura_editor) [file nativename [file join $TEMPURA(tempura_dir) $f]] &
        }
    }
}

# loadfile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#     f           filename
# Results:
#   Returns ...
#
proc loadfile {f} {
    variable TEMPURA
    global tempura_io syntax_error

    set syntax_error 0
    tempura_send "load \"$f\"."
    if { $TEMPURA(watch) != 0 } {
        after cancel $TEMPURA(watch)
    }
    set TEMPURA(mtime) [file mtime $f.t]
    watchfile
    update idletasks
}

# stopexternal --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#     f           filename
# Results:
#   Returns ...
#
proc stopexternal {p} {
    
    catch {exec kill -s SIGTERM [pid $p]} err
    set x 0
    after 100 {set x 1}
    vwait x
    #puts "stopping $p"
    #catch {chan close $p} err
}

# runfile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
#     f            filename
# Results:
#   Returns ...
#
proc runfile {f } {
    variable TEMPURA 
    global tempura_io proggy_iol veri  external runlist

    set lfile [file rootname $TEMPURA(tempura_file)]
    #
    #loadfile $lfile
    #
    #set x 0
    #after 1000 {set x 1}
    #vwait x
    #tempura_send "load \"$lfile\"."

    foreach id [array names proggy_iol] {
        stopexternal $proggy_iol($id)
    }
    
    foreach vr $runlist(plist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            #puts $id
            set ext [lindex $vr 1]
            #puts $ext
            set arg [lrange $vr 2 end]
            #puts $arg
            if {[file exists [file join $TEMPURA(tempura_dir) $ext]$TEMPURA(prog_exe)]} {
                tkterm::e_insert $id "External C program $ext with arguments $arg starts.\n" tempu
                if [string equal $::tcl_platform(platform) "windows"] {
                    if [string equal $::tcl_platform(osVersion) "5.1"] {
                        set tmp1 [file join $TEMPURA(tempura_dir) $ext]$TEMPURA(prog_exe)
                        file copy -force --  $tmp1 $TEMPURA(tempura_path)
                        set tmp [pwd]
                        cd $TEMPURA(tempura_path)
                        set proggy_iol($id) [proggy_spawn [list $ext$TEMPURA(prog_exe) $arg 2>@1] $id]
                        $external($id).pop delete 1
                        $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                        cd $tmp
                    } else {
                        set proggy_iol($id) [proggy_spawn [list $ext$TEMPURA(prog_exe) $arg 2>@1] $id]
                        $external($id).pop delete 1
                        $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    }
                } else {
                    set proggy_iol($id) [proggy_spawn [list [file nativename [file join $TEMPURA(tempura_dir) $ext]$TEMPURA(prog_exe)] $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                }
                set progrun(prun) 1
            } else {
                set progrun(prun) 0
                tkterm::e_insert $id "Oh no, external program $ext does not exist.\n" sta
            }
        } else {
            set progrun(prun) 1
        }
    }

    foreach vr $runlist(analist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set ext [lindex $vr 1]
            if {[file exists $TEMPURA(anatempura)]} {
                tkterm::e_insert $id "local anatempura starts.\n" tempu
                set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(anatempura)] -stdio 2>@1] $id]
                set texit "exit."
                $external($id).pop delete 1
                $external($id).pop add command -label "Terminate external program" -command "catch {proggy_send $proggy_iol($id) $id $texit} err"
                set progrun(anarun) 1
            } else {
                set progrun(anarun) 0
                tkterm::e_insert $id "Oh no, local anatempura $TEMPURA(anatempura) does not exist.\n" sta
            }
        } else {
            set progrun(anarun) 1
        }
    }

    foreach vr $runlist(vlist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set ext [lindex $vr 1]
            if {[file exists [file join $TEMPURA(tempura_dir) $ext]]} {
                if {[file exists $TEMPURA(veriwell)]} {
                    tkterm::e_insert $id "External verilog program $ext starts.\n" tempu
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(veriwell)] -s [file nativename [file join $TEMPURA(tempura_dir) $ext]] 2>@1 ] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(vrun) 1
                } else {
                    tkterm::e_insert $id "Oh no, I can't find the verilog simulator.\n" sta
                }
            } else {
                tkterm::e_insert $id "Oh no, external verilog program $ext does not exist.\n" sta
                set progrun(vrun) 0
            }
        } else {
            set progrun(vrun) 1
        }
    }

    foreach vr $runlist(rmilist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set ext [lindex $vr 1]
            if {[file exists $TEMPURA(rmiregistry)]} {
                tkterm::e_insert $id "rmiregistry starts.\n" tempu
                set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(rmiregistry)] 2>@1] $id]
                set texit "exit."
                $external($id).pop delete 1
                $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                set progrun(rmirun) 1
            } else {
                set progrun(rmirun) 0
                tkterm::e_insert $id "Oh no, rmiregistry $TEMPURA(rmiregistry) does not exist.\n" sta
            }
        } else {
            set progrun(rmirun) 1
        }
    }

    foreach vr $runlist(jlist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set cp [lindex $vr 1]
            set ext [lindex $vr 2]
            set arg [lrange $vr 3 end]
            set cext $ext.class
            set jclass [findinclasspath $ext.class $cp $id]
            if {$jclass != -1} {
                if {[file exists $TEMPURA(java)]} {
                    tkterm::e_insert $id "External java program $ext starts with arguments $arg and using classpath '$cp'.\n" tempu
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(java)] -classpath $cp $ext $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(jrun) 1
                } else {
                   tkterm::e_insert $id "Oh no, the java application launcher does not exist.\n" sta
                   set progrun(jrun) 0
                }
            } else {
                tkterm::e_insert $id "Oh no, I can not find $ext.class in classpath $cp.\n" sta
                set progrun(jrun) 0
            }
        } else {
            set progrun(jrun) 1
        }
    }

    foreach  vr $runlist(rsjlist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            #puts $vr
            set id [lindex $vr 0]
            set cp [lindex $vr 1]
            set ext [lindex $vr 2]
            set arg [lrange $vr 3 end]
            set cext $ext.class
            set pa $TEMPURA(tempura_dir)
            set pa "$pa/"
            
            set jclass [findinclasspath $ext.class $cp $id]
            if {$jclass != -1} {
                if {[file exists $TEMPURA(java)]} {
                    tkterm::e_insert $id "External rmiserver program $ext starts with arguments $arg using classpath '$cp'.\n" tempu
                    set x 0
                    after 300 {set x 1}
                    vwait x
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(java)] -classpath $cp -Djava.rmi.server.codebase=file:$pa $ext $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(rsjrun) 1
                } else {
                   tkterm::e_insert $id "Oh no, the java application launcher does not exist.\n" sta
                   set progrun(rsjrun) 0
                }
            } else {
                tkterm::e_insert $id "Oh no, I can not find $ext.class in classpath $cp.\n" sta
                set progrun(rsjrun) 0
            }
        } else {
            set progrun(rsjrun) 1
        }
    }

    foreach  vr $runlist(rcjlist)  {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            #puts $vr
            set id [lindex $vr 0]
            set cp [lindex $vr 1]
            set ext [lindex $vr 2]
            set arg [lrange $vr 3 end]
            set cext $ext.class
            
            set jclass [findinclasspath $ext.class $cp $id]
            if {$jclass != -1} {
                if {[file exists $TEMPURA(java)]} {
                    tkterm::e_insert $id "External rmiclient program $ext starts with arguments $arg using classpath '$cp'.\n" tempu
                    set x 0
                    after 700 {set x 1}
                    vwait x
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(java)] -classpath $cp $ext $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(rcjrun) 1
                } else {
                   tkterm::e_insert $id "Oh no, the java application launcher does not exist.\n" sta
                   set progrun(rcjrun) 0
                }
            } else {
                tkterm::e_insert $id "Oh no, I can not find $ext.class in classpath $cp.\n" sta
                set progrun(rcjrun) 0
            }
        } else {
            set progrun(rcjrun) 1
        }
    }

    foreach vr $runlist(jjlist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set ext [lindex $vr 1]
            set arg [lrange $vr 2 end]
            set jext $ext.jar
            if {[file exists [file join $TEMPURA(tempura_dir) $jext]]} {
                if {[file exists $TEMPURA(java)]} {
                    tkterm::e_insert $id "External java program $ext starts with arguments $arg.\n" tempu
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(java)] -jar [file nativename [file join $TEMPURA(tempura_dir) $jext]] $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(jjrun) 1
                } else {
                   tkterm::e_insert $id "Oh no, the java application launcher does not exist.\n" sta
                set progrun(jjrun) 0
                }
            } else {
                tkterm::e_insert $id "Oh no, java program $jext does not exist.\n" sta
                set progrun(jjrun) 0
            }
        } else {
            set progrun(jjrun) 1
        }
    }

    
    foreach vr $runlist(cslist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set ext [lindex $vr 1]
            set arg [lrange $vr 2 end]
            if {[file exists [file join $TEMPURA(tempura_dir) $ext]$TEMPURA(csc_exe)]} {
                tkterm::e_insert $id "External C\# program $ext starts with arguments $arg.\n" tempu
                if [string equal $::tcl_platform(platform) "windows"] {
                    set proggy_iol($id) [proggy_spawn [list [file nativename $ext$TEMPURA(csc_exe)] $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(crun) 1
                } else {
                    if {[file exists $TEMPURA(mono)]} {
                        set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(mono)] [file nativename $ext$TEMPURA(csc_exe)] 2>@1] $id]
                        $external($id).pop delete 1
                        $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                        set progrun(crun) 1
                    } else {
                        tkterm::e_insert $id "Oh no, I can't find mono.\n" sta
                        set progrun(crun) 0
                    }
                }
            } else {
                tkterm::e_insert $id "Oh no, I can't find C\# program $ext.\n" sta
                set progrun(crun) 0
            }
        } else {
            set progrun(crun) 1
        }
    }

    foreach vr $runlist(sclist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            set id [lindex $vr 0]
            set cp [lindex $vr 1]
            set ext [lindex $vr 2]
            set arg [lrange $vr 3 end]
            set cext $ext.class
            set scclass [findinclasspath $ext.class $cp $id]
            if {$scclass != -1 } {
                if {[file exists $TEMPURA(scala)]} {
                    tkterm::e_insert $id "External scala program $ext starts with arguments $arg using classpath '$cp'.\n" tempu
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(scala)] -classpath $cp $ext $arg 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(scrun) 1
                } else {
                   tkterm::e_insert $id "Oh no, the scala application launcher does not exist.\n" sta
                set progrun(scrun) 0
                }
            } else {
                tkterm::e_insert $id "Oh no, I can not find $ext.class in classpath $cp.\n" sta
                set progrun(scrun) 0
            }
        } else {
            set progrun(scrun) 1
        }
    }

    foreach vr $runlist(sbtlist) {
        if {$vr!="1"} {
            #set ind [string first " " $vr]
            #set id [string range $vr 0 [expr $ind -1]]
            #set ext [string range $vr [expr $ind +1] end]
            #puts $vr
            set id [lindex $vr 0]
            set ext [lindex $vr 1]
            set arg [lrange $vr 2 end]
            set bext "build.sbt"
            if {[file exists [file join $TEMPURA(tempura_dir) $bext]]} {
                if {[file exists $TEMPURA(sbt)]} {
                    tkterm::e_insert $id "sbt starts $ext with arguments $arg.\n" tempu
                    set proggy_iol($id) [proggy_spawn [list [file nativename $TEMPURA(sbt)] -Dsbt.log.noformat=true "runMain $ext $arg" 2>@1] $id]
                    $external($id).pop delete 1
                    $external($id).pop add command -label "Terminate external program" -command "anatempura::stopexternal $proggy_iol($id)"
                    set progrun(scrun) 1
                } else {
                   tkterm::e_insert $id "Oh no, the sbt application does not exist.\n" sta
                set progrun(sbtrun) 0
                }
            } else {
                tkterm::e_insert $id "Oh no, I can not find build.sbt in $TEMPURA(tempura_dir).\n" sta
                set progrun(scrun) 0
            }  
         } else {
            set progrun(sbtrun) 1
         } 
    }

    set allrun 1
    
    foreach pr [array names progrun] {
        if {$progrun($pr) == 0} then {
            set allrun 0
            break
        }
    }
    
    if { $allrun == 1 } {
        tempura_send "run $f."
        history::addto $TEMPURA(output) "run $f."
        set TEMPURA(rerun) "run $f."
    } else {
        tkterm::o_insert "Oh no, external program did not start.\n" sta
    }
    update idletasks
}

# checkprog --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc checkprog {f id} {
    variable TEMPURA 
    global tempura_io veri

    if {[file exists [file join $TEMPURA(tempura_dir) $f.c]]} {
        if {[file exists $TEMPURA(gcc)]} {
            tkterm::e_insert $id "Compiling external C program.\n" tempu
            set proggy_io [proggy_spawn [list [file nativename $TEMPURA(gcc)] -o [file nativename [file join $TEMPURA(tempura_dir) $f]] [file nativename [file join $TEMPURA(tempura_dir) $f.c]] 2>@1 ] $id]
        } else {
            tkterm::e_insert $id "Oh no, I can't find the C compiler.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, I can't find C program $f.c .\n" sta
    }
}

# checkveri --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc checkveri {f id} {
    variable TEMPURA 
    global tempura_io veri

    if {[file exists [file join $TEMPURA(tempura_dir) $f]]} {
        if {[file exists $TEMPURA(veriwell)]} {
            tkterm::e_insert $id "Compiling external verilog program.\n" tempu
            set proggy_io [proggy_spawn [list [file nativename $TEMPURA(veriwell)] -c [file nativename [file join $TEMPURA(tempura_dir) $f]] 2>@1 ] $id]
        } else {
            tkterm::e_insert $id "Oh no, I can't find the verilog compiler.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, I can't find verilog source $f.\n" sta
    }
}

# checkjava --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc checkjava {f cp id} {
    variable TEMPURA 
    global tempura_io veri

    if {[file exists [file join $TEMPURA(tempura_dir) $f]]} {
        if {[file exists $TEMPURA(javac)]} {
            tkterm::e_insert $id "Compiling external java program.\n" tempu
            set proggy_io [proggy_spawn [list [file nativename $TEMPURA(javac)]  -classpath $cp $f 2>@1] $id]
        } else {
            tkterm::e_insert $id "Oh no, I can't find the java compiler.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, I can't find java source $f.\n" sta
    }
}

# checkjava --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc rmicheckjava {fi f cp id} {
    variable TEMPURA 
    global tempura_io veri

    if {[file exists [file join $TEMPURA(tempura_dir) $fi]]} {
        if {[file exists [file join $TEMPURA(tempura_dir) $f]]} {
            if {[file exists $TEMPURA(javac)]} {
                tkterm::e_insert $id "Compiling external java program.\n" tempu
                set proggy_io [proggy_spawn [list [file nativename $TEMPURA(javac)]  -classpath $cp $fi $f 2>@1] $id]
            } else {
                tkterm::e_insert $id "Oh no, I can't find the java compiler.\n" sta
            }
        } else {
           tkterm::e_insert $id "Oh no, I can't find java source $f.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, I can't find java source $fi.\n" sta
    }
}

# checkcsc --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc checkcsc {f id} {
    variable TEMPURA 
    global tempura_io veri

    if {[file exists [file join $TEMPURA(tempura_dir) $f.cs]]} {
        if {[file exists $TEMPURA(csc)]} {
            tkterm::e_insert "Compiling external C\# program.\n" tempu
            set proggy_io [proggy_spawn [list [file nativename $TEMPURA(csc)] -platform:x86 [file nativename [file join $TEMPURA(tempura_dir) $f.cs]] 2>@1 ] $id]
        } else {
            tkterm::e_insert $id "Oh no, I can't find the C\# compiler.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, I can't find C\# source $f.cs .\n" sta
    }
}

# checkscala --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc checkscala {f cp id} {
    variable TEMPURA 
    global tempura_io veri

    if {[file exists [file join $TEMPURA(tempura_dir) $f]]} {
        if {[file exists $TEMPURA(scalac)]} {
            tkterm::e_insert $id "Compiling external scala program.\n" tempu
            set proggy_io [proggy_spawn [list [file nativename $TEMPURA(scalac)] -classpath $cp $f 2>@1] $id]
        } else {
            tkterm::e_insert $id "Oh no, I can't find the scala compiler.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, I can't find scala source $f.\n" sta
    }
}


}; # end of namespace anatempura


# Namespace visual --
#
#   ADD COMMENTS HERE
#
namespace eval visual {;

    global tempura_io


proc setTooltip {widget text} { 

    if { $text != "" } {
        # 2) Adjusted timings and added key and button bindings. These seem to
        # make artifacts tolerably rare.
        bind $widget <Any-Enter>    [list after 500 [list visual::showTooltip %W $text]]
        bind $widget <Any-Leave>    [list after 500 [list destroy %W.tooltip]] 
        bind $widget <Any-KeyPress> [list after 500 [list destroy %W.tooltip]] 
        bind $widget <Any-Button>   [list after 500 [list destroy %W.tooltip]] 
   } 
}

proc showTooltip {widget text} {
    global tcl_platform
    
    if { [string match $widget* [winfo containing  [winfo pointerx .] [winfo pointery .]] ] == 0  } {
        return 
    } 


   catch { destroy $widget.tooltip } 


   set scrh [winfo screenheight $widget]    ; # 1) flashing window fix 
   set scrw [winfo screenwidth $widget]     ; # 1) flashing window fix 
   set tooltip [toplevel $widget.tooltip -bd 1 -bg black] 
   wm geometry $tooltip +$scrh+$scrw        ; # 1) flashing window fix
   wm overrideredirect $tooltip 1 

   if {$tcl_platform(platform) == {windows}} { ; # 3) wm attributes...
       wm attributes $tooltip -topmost 1   ; # 3) assumes...
   }                                           ; # 3) Windows
   pack [label $tooltip.label -bg lightyellow -fg black -text $text -justify left] 
   set width [winfo reqwidth $tooltip.label] 
   set height [winfo reqheight $tooltip.label] 

   set pointer_below_midline [expr [winfo pointery .] > [expr [winfo screenheight .] / 2.0]]                ; # b.) Is the pointer in the bottom half of the screen?

   set positionX [expr [winfo pointerx .] - round($width / 2.0)]    ; # c.) Tooltip is centred horizontally on pointer.
   set positionY [expr [winfo pointery .] + 35 * ($pointer_below_midline * -2 + 1) - round($height / 2.0)]  ; # b.) Tooltip is displayed above or below depending on pointer Y position.

   # a.) Ad-hockery: Set positionX so the entire tooltip widget will be displayed.
   # c.) Simplified slightly and modified to handle horizontally-centred tooltips and the left screen edge.
   if  {[expr $positionX + $width] > [winfo screenwidth .]} {
       set positionX [expr [winfo screenwidth .] - $width]
   } elseif {$positionX < 0} {
       set positionX 0
   }

   wm geometry $tooltip [join  "$width x $height + $positionX + $positionY" {}] 
   raise $tooltip 

   # 2) Kludge: defeat rare artifact by passing mouse over a tooltip to destroy it.
   bind $widget.tooltip <Any-Enter> {destroy %W} 
   bind $widget.tooltip <Any-Leave> {destroy %W} 
} 


    

# Window --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#    
    proc Window {args} {

        set cmd [lindex $args 0]
        set name [lindex $args 1]
        set newname [lindex $args 2]
        set rest [lrange $args 3 end]

        if {$name == "" || $cmd == ""} {return}
        if {$newname == ""} {
           set newname $name
        }
        set exists [winfo exists $newname]
        switch $cmd {
            show {
                if {$exists == "1" && $name != "."} {wm deiconify $name; return}
                if {[info procs vTclWindow(pre)$name] != ""} {
                    eval "vTclWindow(pre)$name $newname $rest"
                }
                if {[info procs vTclWindow$name] != ""} {
                    eval "vTclWindow$name $newname $rest"
                }
                if {[info procs vTclWindow(post)$name] != ""} {
                    eval "vTclWindow(post)$name $newname $rest"
                }
            }
            hide    { if {$exists} {wm withdraw $newname; return} }
            iconify { if {$exists} {wm iconify $newname; return} }
            destroy { if {$exists} {destroy $newname; return} }
        }
    }


# vTclWindow. --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
    proc vTclWindow. {base} {
        if {$base == ""} {
            set base .
        }
        
        ###################
        # CREATING WIDGETS
        ###################
        #wm focusmodel $base passive
        wm geometry $base 122x26+0+0
        #wm maxsize $base 1137 870
        wm minsize $base 1 1
        wm overrideredirect $base 0
        wm resizable $base 1 1
        wm withdraw $base
        wm title $base "AnaTempura"
     
        ###################
        # SETTING GEOMETRY
        ###################
    }

# GetSelection --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc GetSelection {w} {
	if {
	    ![catch {selection get -displayof $w} txt] ||
	    ![catch {selection get -displayof $w -selection CLIPBOARD} txt]
	} {
	    return $txt
	}
	return -code error "could not find default selection"
}


# vTclWindow.top18 --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc vTclWindow.top18 {base} {

    global tempura_io proggy_iol enter_text entry_widget notebook_widget   
    if {$base == ""} {
        set base .top18
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel  
#    wm focusmodel $base passive
    wm geometry $base 650x510+1+1
    #wm maxsize $base 1300 870
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "AnaTempura:  $anatempura::TEMPURA(file_lbl)"
    wm protocol $base WM_DELETE_WINDOW {
        foreach id [array names proggy_iol] {
           anatempura::stopexternal $proggy_iol($id)
        }
        tempura_send "exit."; exit
    }



    ttk::frame $base.cpd20 \
        -borderwidth 0 -relief raised

    if [string equal $::tcl_platform(os) "Darwin"] {
        menu $base.cpd20.01
    
        $base.cpd20.01 add cascade -label "File" \
           -font $anatempura::TEMPURA(HelvBig) \
           -underline 0 -menu $base.cpd20.01.02

        $base configure -menu $base.cpd20.01

        menu $base.cpd20.01.02 -relief raised -tearoff 0 

        $base.cpd20.01.02 add command \
          -label Exit -font $anatempura::TEMPURA(HelvBig) \
          -command { tempura_send "exit."; exit}
    }   

    ttk::frame $base.cpd21 \
        -borderwidth 1 -relief raised

    ttk::button $base.cpd21.openButton -width 6 -text "OPEN"  \
        -command {anatempura::openfile}

    setTooltip $base.cpd21.openButton "Open Tempura file"

    #ttk::button $base.cpd21.exitButton -width 6 -text "EXIT"  \
    #    -command {tempura_send "exit.";exit}

    ttk::button $base.cpd21.reloadButton -width 6 -text "RELOAD"  \
        -command {anatempura::reload} -state disabled

    setTooltip $base.cpd21.reloadButton "Load Tempura file again"

    ttk::button $base.cpd21.resetButton -width 6 -text "RESET"  \
        -command {anatempura::reset}

    setTooltip $base.cpd21.resetButton "Reset Tempura interpreter"

    ttk::button $base.cpd21.saveButton -width 6 -text "SAVE"  \
        -command {anatempura::savefile}

    setTooltip $base.cpd21.saveButton "Save Tempura output"

    ttk::button $base.cpd21.clearButton -width 6 -text "CLEAR"  \
        -command {tkterm::o_clearnc}

    setTooltip  $base.cpd21.clearButton "Clear Tempura output"   

    ttk::button $base.cpd21.rerunButton -width 6 -text "RE-RUN"  \
        -command {anatempura::rerun} -state disabled

    setTooltip $base.cpd21.rerunButton "Run the last run again"

    if {![string equal $::tcl_platform(platform) "windows"]} {
        ttk::button $base.cpd21.breakButton -width 6 -text "BREAK"  \
            -command {anatempura::break}

        setTooltip $base.cpd21.breakButton "Interrupt run"
    }
    

    ttk::menubutton $base.cpd21.runButton -width 5 -text "RUN"  \
        -menu $base.cpd21.runButton.menu

    setTooltip $base.cpd21.runButton "Select Tempura procedure to run"

    menu $base.cpd21.runButton.menu -relief raised \
        -tearoff 0

    set anatempura::TEMPURA(runmenu) $base.cpd21.runButton.menu

    ttk::menubutton $base.cpd21.editButton -width 5 -text "EDIT"  \
        -menu $base.cpd21.editButton.menu

    setTooltip $base.cpd21.editButton "Select file to edit"

    menu $base.cpd21.editButton.menu -relief raised \
        -tearoff 0

    set anatempura::TEMPURA(editmenu) $base.cpd21.editButton.menu

    ttk::menubutton $base.cpd21.checkButton -width 5 -text "CHECK"  \
        -menu $base.cpd21.checkButton.menu

    setTooltip $base.cpd21.checkButton "Select file to compile/check"
    
    menu $base.cpd21.checkButton.menu -relief raised \
        -tearoff 0

    set anatempura::TEMPURA(checkmenu) $base.cpd21.checkButton.menu    

    ttk::frame $base.cpd22  \
        -borderwidth 1 -height 400 -relief raised \
        -width 169 
    

    ttk::notebook $base.cpd22.n

    set anatempura::TEMPURA(notebook) .top18.cpd22.n
    set notebook_widget $base.cpd22.n

    ttk::frame $base.cpd22.n.f1 
    set frm1 $base.cpd22.n.f1
    $base.cpd22.n add $frm1 -text "Tempura"
    ttk::scrollbar $frm1.04\
        -command [list $frm1.06 xview] -orient horiz  
    ttk::scrollbar $frm1.05 \
        -command [list $frm1.06 yview] -orient vert 
    history::for text $frm1.06 \
        -background $anatempura::TEMPURA(tback) \
        -cursor xterm -borderwidth 1 \
         -highlightthickness 0\
        -relief sunken -font $anatempura::TEMPURA(HelvBig) \
        -wrap char -state normal \
        -height $tkterm::o_rows \
        -insertbackground black \
        -width $tkterm::o_cols \
        -xscrollcommand [list $frm1.04 set]\
        -yscrollcommand [list $frm1.05 set]

    set anatempura::TEMPURA(output)   .top18.cpd22.n.f1.06
    $anatempura::TEMPURA(output) mark set limit insert
    $anatempura::TEMPURA(output) mark gravity limit left

    menu $anatempura::TEMPURA(output).pop
    $anatempura::TEMPURA(output).pop add command -label "Cut"   \
        -command {event generate [focus] <<Cut>>}
    $anatempura::TEMPURA(output).pop add command -label "Copy"   \
        -command {event generate [focus] <<Copy>>}
    $anatempura::TEMPURA(output).pop add command -label "Paste"  \
        -command {event generate [focus] <<Paste>>}
    

    ttk::frame $base.cpd22.n.f3 
    set frm3 $base.cpd22.n.f3
    $base.cpd22.n add $frm3 -text "Help"
    ttk::scrollbar $frm3.04\
        -command [list $frm3.06 xview] -orient horiz  
    ttk::scrollbar $frm3.05 \
        -command [list $frm3.06 yview] -orient vert 
    text $frm3.06 \
        -background $anatempura::TEMPURA(tback) \
        -cursor xterm -borderwidth 1\
        -highlightthickness 0\
        -relief sunken -font $anatempura::TEMPURA(HelvBig)\
        -wrap none \
        -height 22 -insertbackground black \
        -width 80 \
        -xscrollcommand [list $frm3.04 set]\
        -yscrollcommand [list $frm3.05 set]

    set anatempura::TEMPURA(help)     .top18.cpd22.n.f3.06
    helptempura

    ttk::frame $base.cpd22.n.f4 
    set frm4 $base.cpd22.n.f4
    $base.cpd22.n add $frm4 -text "About"   
    ttk::scrollbar $frm4.04 \
        -command [list $frm4.06 xview] -orient horiz  
         ttk::scrollbar $frm4.05 \
        -command [list $frm4.06 yview] -orient vert 

    text $frm4.06 \
        -background $anatempura::TEMPURA(tback) \
        -cursor xterm -borderwidth 1\
        -highlightthickness 0\
        -relief sunken -state disabled\
        -wrap none -font $anatempura::TEMPURA(HelvBig)\
        -height 22 -insertbackground black \
        -width 80 \
        -xscrollcommand [list $frm4.04 set]\
        -yscrollcommand [list $frm4.05 set] 
  
$frm4.06 configure -state normal
$frm4.06 insert end \
"AnaTempura v$anatempura::TEMPURA(version) : GUI for Tempura.

Copyright (C) 1998-2019  Antonio Cau

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

  "
    $frm4.06 configure -state disabled

     

    $anatempura::TEMPURA(output) tag configure tempu -foreground blue
    $anatempura::TEMPURA(output) tag configure sta -foreground red
    $anatempura::TEMPURA(output) tag configure normal -foreground black


    bind $anatempura::TEMPURA(notebook)  <ButtonRelease-1> {+
        update idletasks
    }

    bind all <Control-plus>  [list visual::bump_fonts +2]
    bind all <Control-equal> [list visual::bump_fonts +2]
    bind all <Control-minus> [list visual::bump_fonts -2]
    

    bind $anatempura::TEMPURA(output) <Return> {
    set show_output 0
    history::add? $anatempura::TEMPURA(output);
    set enter_text [$anatempura::TEMPURA(output) get limit end];
    set temp [string range $enter_text 0 2]
    if {$temp == "run"} {
        if {[string range $enter_text end end] == "." } {
            set temp [string trimright $enter_text "."]
        } else {
            set temp $enter_text
        }
        if [catch {$anatempura::TEMPURA(runmenu) index $temp} index] {
            tempura_send "$enter_text"
            set anatempura::TEMPURA(rerun) "$enter_text"
            #set enter_text ""
        } else {
            $anatempura::TEMPURA(runmenu) invoke $index
            #set enter_text ""
        }
    } else {
        tempura_send "$enter_text"
        #set enter_text ""
    }
    $anatempura::TEMPURA(output) insert end \n;
    $anatempura::TEMPURA(output) mark set limit insert;
    break
  }

  if {[tk windowingsystem]=="aqua"} {
    bind $anatempura::TEMPURA(output) <Button-2> {
      tk_popup $anatempura::TEMPURA(output).pop %X %Y
    }
    bind $anatempura::TEMPURA(output) <Control-Button-1> {
      tk_popup $anatempura::TEMPURA(output).pop %X %Y
    }
  } else {
    bind $anatempura::TEMPURA(output) <Button-3> {
      tk_popup $anatempura::TEMPURA(output).pop %X %Y
    }
  }

  bind $anatempura::TEMPURA(output) <ButtonRelease-1> {
    if {[%W compare insert < limit]} {
        %W configure -insertbackground red
    } else {
        %W configure -insertbackground black
    }
  }

  bind $anatempura::TEMPURA(output) <<Paste>> {
    if {![catch {GetSelection %W} txt]} {
        catch {
		if {[%W compare sel.first >= limit]} {
		    %W delete sel.first sel.last
		}
        }
        if {[%W compare insert < limit]} { %W mark set insert end }
        %W configure -insertbackground black
        %W insert insert $txt
	%W see insert
    }
    break
  }

  bind $anatempura::TEMPURA(output) <<Cut>> {
    if {[string match %W [selection own -displayof %W]]} {
	    clipboard clear -displayof %W
	    catch {
		set txt [selection get -displayof %W]
		clipboard append -displayof %W $txt
		if {[%W compare sel.first >= limit]} {
                    %W configure -insertbackground black
		    %W delete sel.first sel.last
		}
	    }
    }
    break
  }

  bind $anatempura::TEMPURA(output) <BackSpace>  {
    if {[%W tag nextrange sel 1.0 end ] != ""} {
        %W delete sel.first sel.last
    } elseif {[%W compare insert > limit]} {
        %W delete insert-1c
        %W see insert
    }
    break
  }

  bind $anatempura::TEMPURA(output) <Key> {
    if [%W compare insert < limit] {
        %W configure -insertbackground black
        %W mark set insert end
    }
  }

  

  bind $anatempura::TEMPURA(output) <Control-c>	{+
    global tempura_io at_input
    if {"%A" != "{}"} {
        if [string equal $::tcl_platform(platform) "windows"] {
             tkterm::o_insert "Oh no, I can't send an interrupt as I'm running under Windows.\n" sta
        } else {
            if [catch {exec kill -s SIGINT [pid $tempura_io]} err] {
                tkterm::o_insert "Oh no, I can't send an interrupt: $err.\n" sta
            } else {
                if { $at_input } {
                    tempura_send "."
                }
            }
        }
    }
}

   bind $anatempura::TEMPURA(output) <KeyRelease> {+
   	update idletasks
   }

   bind $anatempura::TEMPURA(output) <Any-Enter> {+
   	focus %W
   }
 
    ###################
    # SETTING GEOMETRY
    ###################


    pack $base.cpd20 \
        -in $base -anchor center -expand 0 -fill x -side top  

    pack $base.cpd21.openButton -side left -padx 2 -pady 2

    pack $base.cpd21.reloadButton -side left -padx 2 -pady 2

    pack $base.cpd21.rerunButton -side left -padx 2 -pady 2

    pack $base.cpd21.saveButton -side left -padx 2 -pady 2

    pack $base.cpd21.clearButton -side left -padx 2 -pady 2

    pack $base.cpd21.resetButton -side left -padx 2 -pady 2

    if {![string equal $::tcl_platform(platform) "windows"]} {
       pack $base.cpd21.breakButton -side left -padx 2 -pady 2
    }

    #pack $base.cpd21.exitButton -side left -padx 2 -pady 2

    pack $base.cpd21.editButton -side right -padx 2 -pady 2

    pack $base.cpd21.checkButton -side right -padx 2 -pady 2

    pack $base.cpd21.runButton -side right -padx 2 -pady 2

    pack $base.cpd21 -fill x

    pack $base.cpd22 \
        -in $base -anchor center -expand 1 -fill both -side top 


     grid columnconf $base.cpd22 0 -weight 1
     grid rowconf $base.cpd22 0 -weight 1
     grid $base.cpd22.n \
       -in $base.cpd22 -row 0 -column 0 -columnspan 2 -rowspan 1 -sticky nsew
     
    grid columnconf $frm1 0 -weight 1
    grid rowconf $frm1 0 -weight 1
    grid $frm1.04 \
        -in $frm1 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $frm1.05 \
        -in $frm1 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $frm1.06 \
        -in $frm1 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw

    

    grid columnconf $frm3 0 -weight 1
    grid rowconf $frm3 0 -weight 1
    grid $frm3.04 \
        -in $frm3 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $frm3.05 \
        -in $frm3 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $frm3.06 \
        -in $frm3 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw

    grid columnconf $frm4 0 -weight 1
    grid rowconf $frm4 0 -weight 1
    grid $frm4.04 \
        -in $frm4 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $frm4.05 \
        -in $frm4 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $frm4.06 \
        -in $frm4 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw

   

}


proc new_tab {fnr} {
   global external proggy_iol runnames

   set base .top18
   catch { ttk::frame $base.cpd22.n.ef$fnr }
   set frm $base.cpd22.n.ef$fnr
   
    $base.cpd22.n add $frm -text "External $runnames($fnr)"

    catch { ttk::scrollbar $frm.04 \
        -command [list $frm.06 xview] -orient horiz }
    catch { ttk::scrollbar $frm.05 \
        -command [list $frm.06 yview] -orient vert }
    catch { text $frm.06 \
        -background $anatempura::TEMPURA(tback) \
        -cursor xterm -borderwidth 1 \
         -highlightthickness 0\
        -relief sunken -font $anatempura::TEMPURA(HelvBig) \
        -wrap char -state normal \
        -height $tkterm::e_rows \
        -insertbackground black \
        -width $tkterm::e_cols \
        -xscrollcommand [list $frm.04 set] \
        -yscrollcommand [list $frm.05 set]}

    catch { menu $frm.06.pop -tearoff 0}
    
    catch { $frm.06.pop add command -label "Show in separate window"   \
                -command "visual::Window show .top24 .top3$fnr $fnr"            
    }

    
     set external($fnr) .top18.cpd22.n.ef$fnr.06

     $external($fnr) tag configure tempu -foreground blue
     $external($fnr) tag configure sta -foreground red
     $external($fnr) tag configure normal -foreground black

     if {[tk windowingsystem]=="aqua"} {
         bind $frm.06 <Button-2> {
      tk_popup %W.pop %X %Y
    }
    bind $frm.06 <Control-Button-1> {
      tk_popup %W.pop %X %Y
    }
  } else {
    bind $frm.06 <Button-3> {
      tk_popup %W.pop %X %Y
    }
  }
    
     grid $frm.04 \
        -in $frm -column 0 -row 1 -columnspan 3 -rowspan 1 -sticky ew 
     grid $frm.05 \
        -in $frm -column 3 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
     grid $frm.06 \
        -in $frm -column 0 -row 0 -columnspan 3 -rowspan 1 \
        -sticky nesw 
     grid columnconf $frm 0 -weight 1 
     grid columnconf $frm 1 -weight 1 
     grid columnconf $frm 2 -weight 1 
     grid rowconf $frm 0 -weight 1 
    


}



proc vTclWindow.top23 {base} {
    

    if {$base == ""} {
        set base .top23
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel \
        -background $anatempura::TEMPURA(back)
#    wm focusmodel $base passive
    wm geometry $base 730x345+1-1
    # wm maxsize $base 1137 870
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "Simulation"
    ttk::frame $base.cpd22  -borderwidth 1 -height 410 -relief sunken \
        -width 600
    ttk::scrollbar $base.cpd22.01 \
        -command [list $base.cpd22.03 xview] -orient horiz  
    ttk::scrollbar $base.cpd22.02 \
        -command [list $base.cpd22.03 yview] -orient vert 
    canvas $base.cpd22.03 \
        -background $anatempura::TEMPURA(oback) \
        -cursor xterm -borderwidth 1\
         -highlightthickness 0\
        -relief sunken \
        -height 290 -insertbackground black \
        -width 600 \
        -xscrollcommand [list $base.cpd22.01 set] \
        -yscrollcommand [list $base.cpd22.02 set]

    set anatempura::TEMPURA(out)      .top23.cpd22.03


#    ttk::button $base.c \
#    -text "Step" \
#    -command {tkterm::step} -width 4
#    ttk::button $base.d \
#    -text "Start" \
#    -command {tkterm::run} -width 5
#    ttk::button $base.e \
#    -text "Stop" \
#    -command {tkterm::stop} -width 4

    ###################
    # SETTING GEOMETRY
    ###################
    
        
    pack $base.cpd22 \
        -in $base -anchor center -expand 1 -fill both -side top 

#    pack $base.c \
#        -in $base -anchor center -expand 1 -fill y -side left
#    pack $base.d \
#        -in $base -anchor center -expand 1 -fill y -side left 
#    pack $base.e \
#        -in $base -anchor center -expand 1 -fill y -side left
    grid columnconf $base.cpd22 0 -weight 1
    grid rowconf $base.cpd22 0 -weight 1
    grid $base.cpd22.01 \
        -in $base.cpd22 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd22.02 \
        -in $base.cpd22 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $base.cpd22.03 \
        -in $base.cpd22 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
}




proc vTclWindow.top24 {base fnr} {
    global external externalwindow runnames

    if {$base == ""} {
        set base .top24
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel \
        -background $anatempura::TEMPURA(back)
#    wm focusmodel $base passive
    wm geometry $base 450x510+650-1
    # wm maxsize $base 1137 870
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "External Output $runnames($fnr)"
    ttk::frame $base.cpd22  -borderwidth 1 -height 400  -relief raised \
        -width 169
    ttk::scrollbar $base.cpd22.01 \
        -command [list $base.cpd22.03 xview] -orient horiz  
    ttk::scrollbar $base.cpd22.02 \
        -command [list $base.cpd22.03 yview] -orient vert

    $external($fnr) peer create $base.cpd22.03 \
        -background $anatempura::TEMPURA(tback) \
        -cursor xterm -borderwidth 1 \
         -highlightthickness 0\
        -relief sunken -font $anatempura::TEMPURA(HelvBig) \
        -wrap char -state normal \
        -height $tkterm::e_rows \
        -insertbackground black \
        -width $tkterm::e_cols \
        -xscrollcommand [list $base.cpd22.01 set] \
        -yscrollcommand [list $base.cpd22.02 set]

    set externalwindow($fnr) $base.cpd22.03

    ###################
    # SETTING GEOMETRY
    ###################
    
        
    pack $base.cpd22 \
        -in $base -anchor center -expand 1 -fill both -side top 


    grid columnconf $base.cpd22 0 -weight 1
    grid rowconf $base.cpd22 0 -weight 1
    grid $base.cpd22.01 \
        -in $base.cpd22 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd22.02 \
        -in $base.cpd22 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $base.cpd22.03 \
        -in $base.cpd22 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
}



# helptempura --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc helptempura {} {
    variable TEMPURA

$anatempura::TEMPURA(help) configure -state disabled

# invoke --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc invoke {index} {
    variable TEMPURA 
    set tags [$anatempura::TEMPURA(help) tag names $index]
    set i [lsearch -glob $tags demo-*]
    if {$i < 0} {
	return
    }
    set cursor [$anatempura::TEMPURA(help) cget -cursor]
    $anatempura::TEMPURA(help) configure -cursor watch
    update
    set demo [string range [lindex $tags $i] 5 end]
    $anatempura::TEMPURA(help) configure -state normal
    $demo
    update
    $anatempura::TEMPURA(help) configure -cursor $cursor
    $anatempura::TEMPURA(help) configure -state disabled
}

# wnextcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc wnextcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "WNEXT\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `wnext' is the Tempura form of the operator \"weak O\" in temporal
logic. The term \"wnext ...\" causes \"...\" to be executed on the next state, if
there is one. Thus, the term

                           wnext \{ I=1 \}

means that the variable I has the value 1 on the next state if the interval
has a length of at least 1. If the interval is empty, this term is true.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# wherecomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc wherecomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "WHERE\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `where' is equivalent to `and'. It is provided for compatibility
with other systems.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# variablescomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc variablescomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "VARIABLES\n" title
$anatempura::TEMPURA(help) insert end \
"Tempura variables are of two basic kinds:
   -State variables change in value from state to state. Conventionally,
    they are assigned names starting with a capital letter: I, Foo_23, etc.
    The value of a state variable on an INTERVAL is defined to be its value
    on the FIRST STATE. Thus, \"I=0\" means I has initial value zero.
   -Static variables are constant on any given interval. Conventionally,
    they are assigned names starting with a lower-case letter: i, x25, etc.

Variable names may be of unlimited length. Variables and expressions may be
of any of the following types:
   -integer: 0, 42, -97, etc.
   -lambda:  This is the type of predicates and functions.
   -"
$anatempura::TEMPURA(help) insert end "list" {demo demo-listcomm}
$anatempura::TEMPURA(help) insert end ":    Lists may be of fixed or variable length.
   -"
$anatempura::TEMPURA(help) insert end "string" {demo demo-stringcomm}
$anatempura::TEMPURA(help) insert end ":  String constants are of the form \"abc\".
   -"
$anatempura::TEMPURA(help) insert end "float" {demo demo-floatcomm}
$anatempura::TEMPURA(help) insert end ":   Float constants are of the form \$0.123\$ or \$0.123e+10\$.

The function type(X) returns a string representing the type of expression X.
For example, type(9)=\"integer\".

See "
$anatempura::TEMPURA(help) insert end "System" {demo demo-systemcomm}
$anatempura::TEMPURA(help) insert end " for information on system variables.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}


# systemcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc systemcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "SYSTEM VARIABLES\n" title
$anatempura::TEMPURA(help) insert end \
"The following system variables are defined (default values in parentheses):
verbosity    (1)     0 - no display.
                     1 - display statistics at the end of a program only.
                     2 - display memory usage in addition to 1.
                     3 - display statistics after every state.
                     4 - display statistics after every pass.
print_states (true)  If true, all outputs are prefixed with state number.
print_level  (2)     A higher value gives more detail in error messages etc.
random_seed  (0)     The seed (int) used by the \[f\](r|R)andom 
                     number generator, 0 means get seed from clock.
pass_limit   (-1)    A bound on the number of passes per state, -1 indicates 
		     no bound. Normally, there is no reason to use this.
path         (\"\")    The search path used by the load command.
                     If path=\" pgms \\tempura\" the command `load \"file\"' will
                     search for the files: \"file.t\", \"pgms\file.t\" and
                     \"\\tempura\file.t\" (in that order).
infile        (\"stdin\") input redirection
outfile       (\"stdout\") output and format redirection
max_randlen  (10)    Maximum random length interval generated by randlen.
assign_ahead (true)  If true, next values are assigned from the current state. 
break_is_abort (true) If true, an error or break defaults to abort.
hopchop      (true)  If true, evaluate suffix before prefix when not ready.
precision    (5)     Number of digits after `.' used in float output.
They may be set to new values by entering \"set <variable> = <value>.\".

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# stringcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc stringcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "STRINGS\n" title
$anatempura::TEMPURA(help) insert end \
"String constants are enclosed in double quotes, e.g. \"this is a string\".
Most of the `C' escapes may be used. For example '\\n' introduces a newline
character, and '\\\"' introduces a double quote. The character '\\' itself must be
escaped as '\\\\'.

Two strings may be appended with the `+' operator: \"abc\"+\"def\"=\"abcdef\".

Strings may be subscripted in the same way as lists. For example, if
S=\"abcdef\", then S\[2\]=\"c\" and S\[1..4\]=\"bcd\".

The length of a string is denoted by |S|. For example: |\"abcd\"|=4.

The function `ascii' returns the ascii code for the first charater of a string.
For example ascii(\"A\")=65.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# floatcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc floatcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "FLOATS\n" title
$anatempura::TEMPURA(help) insert end \
"Float constants are enclosed in double \$, e.g. \$1.233333\$ or \$1.333e+10\$.

Float operators are: +, -, *, div, mod, **, /, sqrt, ceil, floor, exp, log, 
                     log10, sin, cos, tan, asin, acos, atan, atan2, sinh,
                     cosh, tanh, fabs, itof

To output floats: output(\$2.3333\$) \$2.3333000e+00\$=\$2.3333000e+00\$ 
                  format(\"number is %f\\n\",\$2.3333\$)   number is 2.3333300
                  format(\"number is %e\\n\",\$2.3333\$)   number is 2.3333300e+00

The function `itof' returns the float corresponding to an
integer. Floor and ceil are used to get respectively largest integer
not greater than, smallest integer not less than.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# stablecomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc stablecomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "STABLE\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `stable' corresponds to the notion of stability (constant value).
The term \"stable I\" means that the value of I remains constant throughout the
interval. Note that \"stable I\" is equivalent to \"I gets I\". Thus, I must be
defined on the interval.

One can also use a list of variables, i.e., \"stable(V,W)\" which is equivalent
to \"stable(V) and stable(W)\".

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# sometimescomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc sometimescomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "SOMETIMES\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `sometimes' is the Tempura form of the operator <> in temporal
logic. The term \"sometimes ...\" causes the interpreter to check that  \"...\" is
true at some time in a program. Thus,

          I=0 and I gets I+1 and halt(I=10) and sometimes \{I=5\}

will succeed, whereas

          I=0 and I gets I+1 and halt(I=10) and sometimes \{I=11\}

will fail.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# skipcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc skipcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "SKIP\n" title
$anatempura::TEMPURA(help) insert end \
"The predicate `skip' specifies an interval of length one (containing two
states). Thus,
                      skip and \{I=0\} and next\{I=1\}

defines an interval containing two states. I has value 0 on the first and 1 on
the second.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# semicomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc semicomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "SEMICOLON\n" title
$anatempura::TEMPURA(help) insert end \
"The semicolon operator stands for sequential composition. Intuitively, the term
\"... ; ...\" means \" ... followed by ...\". Thus, the term

       \{len(3) and I=0 and I gets I+1\} ; \{len(4) and I gets I+1\}

is equivalent to

                    \{len(7) and I=0 and I gets I+1\}

The formula before the semicolon must define an interval length. For example,
the term
           \{I=0 and I gets I+1\} ; \{len(4) and I gets I+1\}

is not executable, since it is satisfied by an infinite number of behaviours.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}


# projcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc projcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "PROJ\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `proj' defines temporal projection. By projecting one formula onto
another, we \"insert\" an interval satisfying the first formula between each pair
of states of the second. For example,

            len(2) proj \{len(5) and I=0 and I gets I+1\}

defines an interval of length 10 in which I is incremented on each alternate
state (because an interval of length 2 has been \"inserted\" between each pair of
states of the original formula). Note that the length of the resultant interval
must be an integral multiple of the length of the first formula.

Projection of \"skip\" onto any formula leaves it unchanged.

To execute a program containing projection the system variable \"assign_ahead\"
must be false.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# prefixcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc prefixcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "PREFIX\n" title
$anatempura::TEMPURA(help) insert end \
"The `prefix' of a formula is true on any prefix of an interval which satisfies
the original formula. Thus, for example the formula

                          prefix \{len (10)\}

defines an interval length of at most 10. Therefore, the Tempura program

                   len(2) and prefix \{len(10)\}

defines an interval of length 2.

To execute a program containing `prefix' the system variable \"assign_ahead\"
must be false.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# operatorscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc operatorscomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "OPERATORS\n" title
$anatempura::TEMPURA(help) insert end \
"The following Tempura operators and predicates are built-in:
  true            truth constant              =               equality
  false           truth constant              gets            unit delay
  empty           zero interval length        is              always equals
  skip            unit interval length        <-              assignment
  len ..          any interval length         :=              unit assignment
  and             parallel composition        stable          constant value
  implies         conditional                 more            non empty interval
  if..then..else  conditional                 halt            terminate interval
  exists .. : ..  for local variables         existsf .. : .. for local framed variable
  forall .. : ..  indexed conjunction         fin             finally
  always          the \[\] operator             while..do..     repetition
  sometimes       the <> operator             repeat..until.. repetition
  ;               sequential composition      for..do..       repetition
  next            the O operator              proj            projection
  wnext           the weak O operator         prefix          prefix subinterval
  keep            except on the last state    where           conjunction
  chopstar        repetition                  init            initially
"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# nextcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc nextcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "NEXT\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `next' is the Tempura form of the operator O in temporal logic.
The term \"next ...\" causes \"...\" to be executed on the next state. There
MUST be a next state (cf `wnext'). Thus,

                           next \{ I=1 \}

means that the variable I has the value 1 on the next state. It fails if the
interval is empty.

This operator may also be applied to an expression, yielding its value on the
next state. Thus, the term

                  always \{if more then \{(next I)=J\}\}

is executable. It is equivalent to \"I gets J\".

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# listcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc listcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "LISTS\n" title
$anatempura::TEMPURA(help) insert end \
"A Tempura list may have either fixed (static), or variable (state-varying)
length. These are defined by means of the predicates `list' and `List'.

    list(L,n) defines L to be a list of fixed size n,
    List(L,n) defines L to be a list of variable size up to a maximum of n.

These predicates apply ON THE CURRENT STATE ONLY. There is an additional predicate 
`stable(struct(L))', which keeps the list structure of variable `L' stable. So, for
example, \"List(L,5) and stable(struct(L))\" defines L to always be a list of maximum
length 5. The expression \[1,2,3,4,5\] is a list constant, with elements 1,2,3...

To access an element of a list, the notation `L\[i\]' should be used. Indexing
begins at zero. Thus, for example, if L=\[1,2,3,4\] then L\[2\]=3.

A similar notation is used to access a sublist of a list. `L\[i..j\]' denotes
a sublist of L from i to j-1. For example, \"list(L,5) and L\[1..3\]=\[1,2\]\"
assigns the values 1 and 2 to the second and third elements of the list.
The expression |L| returns the (integer) size of the list L.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# lencomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc lencomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "LEN\n" title
$anatempura::TEMPURA(help) insert end \
"The `len' construct can be used to define any interval length. The term \"len(n)\"
defines an interval of length n (containing n+1 states). Thus,

                          len(0) = empty
                          len(1) = skip

The `randlen' construct can be used to generate a random length
(greater or equal than zero and less or equal than max_randlen) interval.
"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}


# keepcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc keepcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "KEEP\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `keep' has the intuitive meaning: \"for all unit intervals do\". 
This operator is defined as follows:

           keep w  =def  always \{if more then w\}  w is state formula
           keep f  =def   box-a \{if skip then f\}

Thus, the term

                        keep \{(next I)=J\}

is equivalent to \"I gets J\".

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# iscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc iscomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "IS\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `is' corresponds to zero delay. The term \"I is J\" means that
the values of I and J are always equal. The term \"I is J\" is equivalent to

                        always \{I=J\}

This defines a whole sequence of values for I and J.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# initcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc initcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "INIT\n" title
$anatempura::TEMPURA(help) insert end \
"The term \"init ...\" means that \"...\" must be true initially. The `init'
operator not needed in Tempura programs, because it is redundant. It is
provided for compatibility with other interpreters.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# itecomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc itecomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "IF ... THEN ... ELSE ...\n" title
$anatempura::TEMPURA(help) insert end "IF ... THEN ... ELSEIF ... ... ELSEIF ... THEN ... ELSE ...\n" title
$anatempura::TEMPURA(help) insert end \
"The notation \"if ... then ... else ...\" represents a binary choice. This
notation may be used for both statements and expressions. For example, the term
\"A gets B\" is equivalent to

                always \{if more then (next A)=B\}

The form \"if ... then ...\" is the same as \"if ... then ... else true\" 
and is the same as \" ... implies ...\".

The expression
                 A=0 and B=(if A=1 then 3 else 4)

assigns an initial value of 4 to B.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# haltcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc haltcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "HALT, FIN\n" title
$anatempura::TEMPURA(help) insert end \
"The `halt' operator defines the termination condition for an interval. For
example, the formula:

                   I=0 and I gets I+1 and halt(I=5)

defines an interval on which I is incrememted state-by-state until it equals 5.
The interval length must therefore be 5. An equivalent formula is:

                   I=0 and I gets I+1 and len 5

The term \"fin ...\" means that \"...\" must be true on the last state of the
interval. Thus, the formula

                  len(5) and keep \{I=0\} and fin \{I=1\}

defines an interval of length 5 for which I is 1 on the last state, and
zero on every other state. Be aware that `fin' is NOT the same as `halt'.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# getscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc getscomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "GETS\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `gets' corresponds to unit delay. The term \"I gets J\" means that
the value of I on the next state is always equal to the value of J on the
current state. Note that \"I gets J\" is equivalent to

                        keep \{(next I)=J\}

This defines a whole sequence of values for I.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# forallcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc forallcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "FORALL\n" title
$anatempura::TEMPURA(help) insert end \
"Universal quantification is denoted by the `forall' operator. Only the bounded form

                      forall i<n:\{ ... \}

is permitted in Tempura. Semantically, universal quantification corresponds to
\"indexed concurrency\". The term

     forall i<n:\{ p(i) \}    is equivalent to    p(0) and ... and p(n-1).

For example,

                      forall i<|L|:\{ L\[i\]=0 \}

sets all the elements of the list L to zero on the first state.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# whilecomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc whilecomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "WHILE, REPEAT, CHOPSTAR and FOR\n" title
$anatempura::TEMPURA(help) insert end \
"The `while', `repeat' and `for' operators define sequential loops in the usual
way. The following forms may be used:

             while <condition> do \{ ... \}
             repeat \{ ... \} until <expression>
             chopstar \{ ... \}
             for <variable> < <integer expression> do \{ ... \}
             for <variable> in <list/string expression> do \{ ... \}
             for <integer expression> times do \{ ... \}

`chopstar \{ ... \}' is equal to `while true do \{ ... \}'.

For example, the formulae

             lis=\[0,1,2\] and for i<3 do \{skip and output(lis\[i\])\}
and
             lis=\[0,1,2\] and for e in lis do \{skip and output(e)\}

are equivalent. Note that the control variable of a `for' loop should be
static.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# fincomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc fincomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "FIN\n" title
$anatempura::TEMPURA(help) insert end \
"The term \"fin ...\" means that \"...\" must be true on the last state of the
interval. Thus, the formula

                  len(5) and keep \{I=0\} and fin \{I=1\}

defines an interval of length 5 for which I is 1 on the last state, and
zero on every other state. Be aware that `fin' is NOT the same as `halt'.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# exprcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc exprcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "EXPRESSIONS\n" title
$anatempura::TEMPURA(help) insert end \
"Expressions are composed of variables, constants and operators. The following
is a list of operators which may occur within expressions:

true             truth constant           \$..\$             float constants
false            truth constant           \"..\"             string constants
empty            zero interval length     ascii(..)        code for string char
and              logical conjunction      \[..\]             lists and subscripts
or               logical disjunction      |..|             list size
not              logical negation         f(..)            function call
if..then..else   conditional              lambda(..) : ..  lambda expression
exists .. : ..   existence                ref(..)          address of
forall .. : ..   indexed conjunction      deref(..)        contents of
next(..)         on the next state        type(..)         type of
prev(..)         on the previous state    nprev(k,..)      on the k-previous state
.. defaults ..   default value            more             non empty interval
=,~=,<,<=,>=,>   relational operators     
+,-,*,div,mod,**,/,sqrt,ceil,floor,exp,log,log10,itof,
sin,cos,tan,asin,acos,atan,atan2,sinh,cosh,tanh,fabs    arithmetic operators
random           integer random  (static within a state)
Random           integer random  (can change within a state)
frandom          float random number in \[\$0.0\$, \$1.0\$) (static within a state)
fRandom          float random number in \[\$0.0\$, \$1.0\$) (can change within a state)
State_number     the number of the current state (read_only)

See "
$anatempura::TEMPURA(help) insert end "Variables" {demo demo-variablescomm}
$anatempura::TEMPURA(help) insert end " for a description of variables and their types. "
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# existscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc existscomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "EXISTS\n" title
$anatempura::TEMPURA(help) insert end \
"Existential quantification is denoted in Tempura by the `exists' operator.
Semantically, existential quantification corresponds to the introduction of
local variables. For example, in the term

        exists I,J:\{I=0 and J=1 and exists I:\{I=J and ...\} and ...\}

there are two instances of the variable I. The outer one has initial value 0,
the inner one has initial value 1.

Optionally one can specify the length of the history with #n (n and integer >0)
that is to be kept for a variable. The values in the history can be accessed 
by prev and nprev operators. When history is omitted a default of #1 is taken.

        exists X #2 : \{ X=0 and len(3) and X gets X+1 and always
                        (output(nprev(X,2))) \}

The bounded existential quantification:

                       exists i<|L|:( L\[i\]=0 )
is equivalent to
                       not (forall i<|L|:not (L\[i\]=0))

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# existscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc existsfcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "EXISTSF\n" title
$anatempura::TEMPURA(help) insert end \
"Semantically, existential framed quantification corresponds to the
introduction of local framed variables. For example, in the term

        existsf I:\{I=0 and skip and always output I\}

the value of the variable I is initially 0 and retains that value in the
second state. This in contrast with the term

        exists I:\{I=0 and skip and always output I\}

where the value of the variable I is initially 0 and undefined in the
second state.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}


# equalcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc equalcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "EQUALITY\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `=' defines equality in the usual way. However, because the value
of a state variable is defined to be its value on the first state of an
interval, the term \"I=0\" only means that I has value zero ON THE FIRST STATE.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# emptycomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc emptycomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "EMPTY and MORE\n" title
$anatempura::TEMPURA(help) insert end \
"The predicate `empty' specifies an interval of length zero (containing just one
state). Thus,
                           empty and I=0

defines an interval containing one state on which I has the value zero.

The predicate `more' is defined as the non zero interval, i.e., empty = not more.

One may also use `empty' or 'more' as an expression. For example, the term

                     always \{if more
                             then output(\"not the last state\")
                             else output(\"the last state\")\}

prints out whether or not each state is the final state of an interval.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# assigncomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc assigncomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "ASSIGNMENT\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `<-' corresponds to assignment. The term \"I <- J\" means that
the value of I on the last state is equal to the value of J on the first
state. Note that \"I <- J\" is equivalent to

                     exists x:\{ x=J and fin \{I=x\}\}

This defines a single value for I.

"

$anatempura::TEMPURA(help) insert end "UNIT ASSIGNMENT\n" title
$anatempura::TEMPURA(help) insert end \
"The operator `:=' corresponds to unit assignment. The term \"I := J\" means that
the value of I on the next state is equal to the value of J on the current
state. Note that \"I := J\" is equivalent to \"(next I) = J\".

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# andcomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc andcomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "AND\n" title
$anatempura::TEMPURA(help) insert end \
"The logical conjunction operator `and' may be used to compose formulae in parallel. 
For example, the conjunction   

     len 5 and always \{I=0\} and always \{J=1\}

causes the terms \"len 5\", \"always \{I=0\}\", and \"always \{J=1\}\" to be executed concurrently. 
That is, it defines a new interval of length 5 on which I is always 0 and J is always 1.

Boolean expressions may also be combined with `and' in the usual way:

           A=true and B=false and C=(A and B)

sets C to false.

"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# alwayscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc alwayscomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "ALWAYS\n" title

$anatempura::TEMPURA(help) insert end \
"The operator `always' is the Tempura form of the operator \[\] in temporal logic.
The term \"always ...\" causes \"...\" to be executed on every state. Thus,

                           always \{ I=1 \}

means that the variable I always has the value 1. Note that a term such as

                           always \{ next \{ I=1 \}\}

cannot be successfully executed, since the `next' operator implies an interval
having more than one state, and this cannot be executed on the last state of an
interval. However, the term

                  always \{if more then \{next \{I=1\}\}\}

is executable. It is equivalent to \"I gets 1\".
 
"
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}

# commandscomm --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc commandscomm {} {
  variable TEMPURA 

$anatempura::TEMPURA(help) delete 1.0 end

$anatempura::TEMPURA(help) insert end "SYSTEM COMMANDS\n" title

$anatempura::TEMPURA(help) insert end "The following system commands are available:\n" 
$anatempura::TEMPURA(help) insert end " \n " 

$anatempura::TEMPURA(help) insert end \
" load \"file\".              Execute the commands in \"file.t\" (note extension).
  define <f/p> = <body>.    Define a function/procedure. 
  run <program>.            Run a Tempura program.
  input <expression>.       Input the value of an expression.
  output <expn list>.       Output the value of expressions.
  peek <expression>.        Output expression (or \? if undefined).
  format (<expn list>).     Formatted output.
  set <sysid> = <value>.    Set a system variable (see "
$anatempura::TEMPURA(help) insert end "System" {demo demo-systemcomm}
$anatempura::TEMPURA(help) insert end ").
  break.                    Call the interpreter recursively (from a program).
  showtrace.                Display the execution stack and environment.
  showstate.                Display what variables are (un)defined in
	                    the current state.
  continue.                 Continue execution (following break).
  step.                     Advance execution one step (following break).
  jump.                     Step to next statement at current reduction depth.
  return.                   Exit from a recursive call of the interpreter.
  abort.                    Return to the outermost level of Tempura.
  exit.                     Exit from the Tempura system.
  
  "
$anatempura::TEMPURA(help) insert end "Index" {demo demo-indexcomm}

}


$anatempura::TEMPURA(help) tag configure title -foreground red -font $anatempura::TEMPURA(HelvBig)

$anatempura::TEMPURA(help) tag configure demospace -lmargin1 1c -lmargin2 1c
if {[winfo depth .] == 1} {
    $anatempura::TEMPURA(help) tag configure demo -lmargin1 1c -lmargin2 1c \
	-underline 1 -tabs 
    $anatempura::TEMPURA(help) tag configure visited -lmargin1 1c -lmargin2 1c \
	-underline 1

} else {
    $anatempura::TEMPURA(help) tag configure demo -lmargin1 1c -lmargin2 1c \
	-foreground  blue -underline 1
    $anatempura::TEMPURA(help) tag configure visited -lmargin1 1c -lmargin2 1c \
	-foreground #303080 -underline 1

}
$anatempura::TEMPURA(help) tag bind demo <ButtonRelease-1> {
    visual::invoke [$anatempura::TEMPURA(help) index {@%x,%y}]
}
set lastLine ""
$anatempura::TEMPURA(help) tag bind demo <Enter> {
    set lastLine [$anatempura::TEMPURA(help) index {@%x,%y linestart}]
    $anatempura::TEMPURA(help) config -cursor hand2
}
$anatempura::TEMPURA(help) tag bind demo <Leave> {
    $anatempura::TEMPURA(help) config -cursor xterm
}
$anatempura::TEMPURA(help) tag bind demo <Motion> {
    set newLine [$anatempura::TEMPURA(help) index {@%x,%y linestart}]
    if {[string compare $newLine $lastLine] != 0} {
	set lastLine $newLine

	set tags [$anatempura::TEMPURA(help) tag names {@%x,%y}]
	set i [lsearch -glob $tags demo-*]
	if {$i >= 0} {
	}
    }
}


# indexcomm --
#
#   ADD COMMENTS HERE
#   Create the text for the text widget.
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc indexcomm {} {
  variable TEMPURA
$anatempura::TEMPURA(help) delete 1.0 end
$anatempura::TEMPURA(help) insert end "Tempura Help Index\n" title

$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Commands" {demo demo-commandscomm}
$anatempura::TEMPURA(help) insert end           "              " {demospace}
$anatempura::TEMPURA(help) insert end "Halt, Fin" {demo demo-haltcomm}
$anatempura::TEMPURA(help) insert end            "             " {demospace}
$anatempura::TEMPURA(help) insert end "Projection" {demo demo-projcomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Always" {demo demo-alwayscomm}
$anatempura::TEMPURA(help) insert end         "                " {demospace}
$anatempura::TEMPURA(help) insert end "Choice" {demo demo-itecomm}
$anatempura::TEMPURA(help) insert end         "                " {demospace}
$anatempura::TEMPURA(help) insert end "Semicolon" {demo demo-semicomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "And" {demo demo-andcomm}
$anatempura::TEMPURA(help) insert end      "                   " {demospace}
$anatempura::TEMPURA(help) insert end "Init" {demo demo-initcomm}
$anatempura::TEMPURA(help) insert end       "                  " {demospace}
$anatempura::TEMPURA(help) insert end "Skip" {demo demo-skipcomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Assignment" {demo demo-assigncomm}
$anatempura::TEMPURA(help) insert end             "            " {demospace}
$anatempura::TEMPURA(help) insert end "Is" {demo demo-iscomm}
$anatempura::TEMPURA(help) insert end     "                    " {demospace}
$anatempura::TEMPURA(help) insert end "Sometimes" {demo demo-sometimescomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Empty" {demo demo-emptycomm}
$anatempura::TEMPURA(help) insert end        "                 " {demospace}
$anatempura::TEMPURA(help) insert end "Keep" {demo demo-keepcomm}
$anatempura::TEMPURA(help) insert end       "                  " {demospace}
$anatempura::TEMPURA(help) insert end "Stable" {demo demo-stablecomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Exists" {demo demo-existscomm}
$anatempura::TEMPURA(help) insert end         "                " {demospace}
$anatempura::TEMPURA(help) insert end "Len" {demo demo-lencomm}
$anatempura::TEMPURA(help) insert end      "                   " {demospace}
$anatempura::TEMPURA(help) insert end "String" {demo demo-stringcomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Expression" {demo demo-exprcomm}
$anatempura::TEMPURA(help) insert end             "            " {demospace}
$anatempura::TEMPURA(help) insert end "List" {demo demo-listcomm}
$anatempura::TEMPURA(help) insert end       "                  " {demospace}
$anatempura::TEMPURA(help) insert end "System" {demo demo-systemcomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Loop" {demo demo-whilecomm}
$anatempura::TEMPURA(help) insert end       "                  " {demospace}
$anatempura::TEMPURA(help) insert end "Next" {demo demo-nextcomm}
$anatempura::TEMPURA(help) insert end       "                  " {demospace}
$anatempura::TEMPURA(help) insert end "Variables" {demo demo-variablescomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Forall" {demo demo-forallcomm}
$anatempura::TEMPURA(help) insert end         "                " {demospace}
$anatempura::TEMPURA(help) insert end "Operators" {demo demo-operatorscomm}
$anatempura::TEMPURA(help) insert end            "             " {demospace}
$anatempura::TEMPURA(help) insert end "Where" {demo demo-wherecomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Gets" {demo demo-getscomm}
$anatempura::TEMPURA(help) insert end       "                  " {demospace}
$anatempura::TEMPURA(help) insert end "Prefix" {demo demo-prefixcomm}
$anatempura::TEMPURA(help) insert end         "                " {demospace}
$anatempura::TEMPURA(help) insert end "Wnext" {demo demo-wnextcomm}
$anatempura::TEMPURA(help) insert end " \n " {demospace}
$anatempura::TEMPURA(help) insert end "Floats" {demo demo-floatcomm}
$anatempura::TEMPURA(help) insert end         "                " {demospace}
$anatempura::TEMPURA(help) insert end "Existsf" {demo demo-existsfcomm}
}


$anatempura::TEMPURA(help) configure -state normal
indexcomm
$anatempura::TEMPURA(help) configure -state disabled
}

proc bump_fonts {incr} {
    catch {
      set size [font configure $anatempura::TEMPURA(HelvBig) -size]
      incr size $incr
      if {$size >= 8 && $size <= 32} {
        font configure $anatempura::TEMPURA(HelvBig) -size $size
      }
    }
}

}; # end of namespace visual



# Namespace tkterm --
#
#   ADD COMMENTS HERE
#
namespace eval tkterm {;      

   variable o_term
   variable o_rows           
   variable o_cols
    
   variable stepper
   variable run

   variable e_term 
   variable e_rows          
   variable e_cols          


# run --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc run {} {
  variable stepper
  variable run

  set run 1
  set stepper $stepper
}

# stop --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc stop {} {
  variable stepper
  variable run

  set run 0
  set stepper $stepper
}


# step --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc step {} {
  variable stepper
  variable run

 set stepper 1
 set run 0

}


# break --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc tbreak {} {
  variable  stepper
  variable  run

  update 
    #  puts "run:$run"
    #  puts "stepper:$stepper"
    if {$run == 0} {
        tkwait variable tkterm::stepper
    }

}


# o_clearnc --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc o_clearnc {} {
    variable o_term 
        
    $o_term delete 1.0 {end -1 lines}
    $o_term mark set limit insert
    update idletasks     
}

# o_insert --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc o_insert {s {tag normal}} {
    variable o_term
    global stdio

    $o_term mark set insert end
    $o_term insert insert $s $tag
    
    if {[$o_term compare insert != "insert linestart"]} {
        if {$stdio == 1} { puts $s}
        $o_term insert insert \n
    } else {
        if {$stdio == 1} { puts -nonewline $s }
    }
    $o_term see insert
    $o_term mark set limit insert
    update idletasks   
}


# e_insert --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc e_insert {id s {tag normal}} {
    variable e_term
    global external externalwindow stdio

    if {![info exists external($id)] || $stdio == 1} {
        o_insert $s $tag
    } else {
        $external($id) insert end $s $tag
        $external($id) see end
    }
    if {[winfo exists .top3$id]} {
        $externalwindow($id) see end
    }
    update idletasks   
}

}; # end of namespace tkterm


# Namespace tkproggy --
#
#   ADD COMMENTS HERE
#
namespace eval tkproggy {;
  global tempura_io proggy_iol

# start --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc start {id nl} {
    global proggy_iol
   
    set tt [join $nl " "]
    if {[info exists proggy_iol($id)]} {
        if {[catch {proggy_send $proggy_iol($id) $id "$tt"} err]} {
            tkterm::e_insert $id "Oh no, possible error sending $tt to process $id: $err, data is buffered.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, error sending $tt to process $id: process not started.\n" sta
    }
    update idletasks 
}

# startveri --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc startveri {id nl} {
  global proggy_iol
    if {[info exists proggy_iol($id)]} {
        if {[catch {proggy_send $proggy_iol($id) $id "$nl;"} err]} {
            tkterm::e_insert $id "Oh no, possible error sending $nl; to process $id: $err, data is buffered.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, error sending $nl; to process $id: process not started.\n" sta
    }
    update idletasks 
}

# startdot --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc startdot {id} {
  global proggy_iol
    if {[info exists proggy_iol($id)]} {
        if {[catch {proggy_send $proggy_iol($id) $id "."} err]} {
            tkterm::e_insert $id "Oh no, possible error sending . to process $id: $err, data is buffered.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, error sending . to process $id: process not started.\n" sta
    }
    update idletasks 
}

# startfinish --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc startfinish {id} {
  global proggy_iol
    if {[info exists proggy_iol($id)]} {
        if {[catch {proggy_send $proggy_iol($id) $id "\$finish;"} err]} {
            tkterm::e_insert $id "Oh no, possible error sending \$finish; to process $id: $err, data is buffered.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, error sending \$finish; to process $id: process not started.\n" sta
    }
    update idletasks 
}

# startstop --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc startstop {id time} {
  global proggy_iol
    if {[info exists proggy_iol($id)]} {
        if {[catch {proggy_send $proggy_iol($id) $id "#$time \$stop;"} err]} {
            tkterm::e_insert $id "Oh no, possible error sending #$time \$stop; to process $id: $err, data is buffered.\n" sta
        }
    } else {
        tkterm::e_insert $id "Oh no, error sending #$time \$stop; to process $id: process not started.\n" sta
    }
    update idletasks 
}

# tbreak --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc tbreak {id nl} {
   global tempura_io at_prompt at_input proggy_iol stdio

   set vname [lindex $nl 0]
   set totnl [lrange $nl 1 end-1]
   set tot ""
   foreach i $totnl {
    set tottemp [string trimleft $i]
    set totstring ",\"$tottemp\""
    set tot $tot$totstring
   }
    set tot "\[\"$vname\"$tot\].\r"

    #puts "at_prompt: $at_prompt"
    #puts "at_input: $at_input"
    #puts "stdio: $stdio"

    if {!$at_prompt || $at_input || $stdio} {
        tempura_send $tot
    } else {
        #puts "loose $tot"
        #tkterm::o_insert "Oh no, no matching input, stopping external program.\n" sta
        #anatempura::stopexternal $proggy_iol($id)
    }
   
   update idletasks
}

}; # end of namespace tkproggy


# init --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc init {argc argv} {
    global env stdio

    if [string equal $::tcl_platform(platform) "windows"] {
        set anatempura::TEMPURA(newline)         ""
        set anatempura::TEMPURA(tempura_editor)  "c:/Program Files/Windows NT/Accessories/wordpad.exe"
        set anatempura::TEMPURA(HelvBig)         {TkFixedFont}
        set anatempura::TEMPURA(tempura_dir)     [file join [pwd] Examples]
        if [string equal $::tcl_platform(osVersion) "5.1"] {
            set anatempura::TEMPURA(sbt)            "c:/Program Files (x86)/sbt/bin/sbt.bat"
            set anatempura::TEMPURA(scala)            "c:/Program Files (x86)/scala/bin/scala.bat"
            set anatempura::TEMPURA(scalac)           "c:/Program Files (x86)/scala/bin/scalac.bat"
            set anatempura::TEMPURA(java)            "c:/Program Files (x86)/Java/jdk1.8.0_181/bin/java.exe"
            set anatempura::TEMPURA(javac)           "c:/Program Files (x86)/Java/jdk1.8.0_181/bin/javac.exe"
            set anatempura::TEMPURA(rmiregistry)     "c:/Program Files (x86)/Java/jdk1.8.0_181/bin/rmiregistry.exe"
            set anatempura::TEMPURA(csc)             "c:/Windows/Microsoft.NET/Framework/v2.0.50727/csc.exe"
        } else {
            set anatempura::TEMPURA(sbt)            "c:/Program Files (x86)/sbt/bin/sbt.bat"
            set anatempura::TEMPURA(scala)            "c:/Program Files (x86)/scala/bin/scala.bat"
            set anatempura::TEMPURA(scalac)           "c:/Program Files (x86)/scala/bin/scalac.bat"
            set anatempura::TEMPURA(java)            "c:/Program Files/Java/jdk1.8.0_181/bin/java.exe"
            set anatempura::TEMPURA(javac)           "c:/Program Files/Java/jdk1.8.0_181/bin/javac.exe"
            set anatempura::TEMPURA(rmiregistry)     "c:/Program Files/Java/jdk1.8.0_181/bin/rmiregistry.exe"
            set anatempura::TEMPURA(csc)             "c:/Windows/Microsoft.NET/Framework/v2.0.50727/csc.exe"
        }
        set anatempura::TEMPURA(veriwell)        ""
        set anatempura::TEMPURA(tempura_path)    [pwd]
        if {[info exists ::starkit::topdir]} {
            set tmpf [file tail $::starkit::topdir]
            if { $tmpf == "anatempura-win64.exe" } {
                set anatempura::TEMPURA(gcc)             "/mingw64/bin/gcc.exe"
                set anatempura::TEMPURA(tempura_exe)     "tempura-win64.exe"
            } else {
                set anatempura::TEMPURA(gcc)             "/ming32/bin/gcc.exe"
                set anatempura::TEMPURA(tempura_exe)     "tempura.exe"
            }
        } else {
            set tmpf [lindex [exec wmic computersystem get systemtype] 1]
            if {  $tmpf == "x64-based"} {
                set anatempura::TEMPURA(gcc)             "/mingw64/bin/gcc.exe"
                set anatempura::TEMPURA(tempura_exe)     "tempura-win64.exe"
            } else {
                set anatempura::TEMPURA(gcc)             "/mingw32/bin/gcc.exe"
                set anatempura::TEMPURA(tempura_exe)     "tempura.exe"
            }
        }
        set anatempura::TEMPURA(rc_file)         ".anatempurarc"
        set anatempura::TEMPURA(prog_exe)        ".exe"
        set anatempura::TEMPURA(csc_exe)         ".exe"
    } else {
        set anatempura::TEMPURA(wave)            "gtkwave"
        set anatempura::TEMPURA(rc_file)         ".anatempurarc"
        set anatempura::TEMPURA(newline)         "\r"
        set anatempura::TEMPURA(HelvBig)         {TkFixedFont}
        set anatempura::TEMPURA(tempura_dir)     [file join [pwd] Examples]
        set anatempura::TEMPURA(java)            "/usr/bin/java"
        set anatempura::TEMPURA(javac)           "/usr/bin/javac"
        set anatempura::TEMPURA(rmiregistry)     "/usr/bin/rmiregistry"
        set anatempura::TEMPURA(sbt)             "/usr/bin/sbt"
        if [string equal $::tcl_platform(os) "SunOS"] {
            set anatempura::TEMPURA(tempura_editor)  "vi"
            set anatempura::TEMPURA(gcc)             "/opt/csw/gcc4/bin/gcc"
            set anatempura::TEMPURA(tempura_exe)     "tempura_solaris"
            set anatempura::TEMPURA(tempura_path)     [pwd]
            set anatempura::TEMPURA(csc_exe)         ".exe"
            set anatempura::TEMPURA(scala)         "/usr/bin/scala"
            set anatempura::TEMPURA(scalac)         "/usr/bin/scalac"
        } elseif {[string equal $::tcl_platform(os) "Linux"]} {
            set anatempura::TEMPURA(tempura_editor)  "gedit"
            set anatempura::TEMPURA(gcc)           "/usr/bin/gcc"
            set anatempura::TEMPURA(mono)          "/usr/bin/mono"   
            set anatempura::TEMPURA(csc)           "/usr/bin/mcs"
            set anatempura::TEMPURA(scala)         "/usr/bin/scala"
            set anatempura::TEMPURA(scalac)         "/usr/bin/scalac"
            if [string equal $::tcl_platform(machine) "x86_64"] {
                set anatempura::TEMPURA(tempura_exe)     "tempura_linux64"
            } else {
                set anatempura::TEMPURA(tempura_exe)     "tempura_linux"
            }
            set anatempura::TEMPURA(tempura_path)     [pwd]
            set anatempura::TEMPURA(csc_exe)         ".exe"
        } elseif {[string equal $::tcl_platform(os) "Darwin"]} {
            set anatempura::TEMPURA(gcc)           "/usr/bin/gcc"
            set anatempura::TEMPURA(csc)           "/opt/local/bin/mcs"
            set anatempura::TEMPURA(mono)          "/opt/local/bin/mono"
            set anatempura::TEMPURA(tempura_editor)  "/usr/bin/emacs"
            set anatempura::TEMPURA(javac)          "/usr/bin/javac"
            set anatempura::TEMPURA(java)          "/usr/bin/java"
            set anatempura::TEMPURA(rmiregistry)   "/usr/bin/rmiregistry"
            set anatempura::TEMPURA(veriwell)      "/opt/local/bin/veriwell"
            set anatempura::TEMPURA(scala)         "/opt/local/bin/scala-2.12"
            set anatempura::TEMPURA(scalac)         "/opt/local/bin/scalac-2.12"
            set anatempura::TEMPURA(sbt)             "/opt/local/bin/sbt"
            set anatempura::TEMPURA(tempura_editor)  "/usr/bin/emacs"
            set anatempura::TEMPURA(tempura_exe)     "tempura_macosx"
            set anatempura::TEMPURA(tempura_path)     [pwd]
            set anatempura::TEMPURA(csc_exe)         ".exe"
        } else {
            puts "Platform not supported yet but will continue anyway..."
            set anatempura::TEMPURA(tempura_editor)  "vi"
            set anatempura::TEMPURA(gcc)           "/usr/bin/gcc"
            set anatempura::TEMPURA(tempura_exe)     "tempura"
            set anatempura::TEMPURA(tempura_path)     [pwd]
        }
        #set anatempura::TEMPURA(veriwell)        "/usr/local/bin/veriwell"
        set anatempura::TEMPURA(prog_exe)        ""
    }
    #set anatempura::TEMPURA(fore)     black
    set anatempura::TEMPURA(back)     #99aecb
    tk_setPalette $anatempura::TEMPURA(back)
    set anatempura::TEMPURA(tback)    white
    set anatempura::TEMPURA(oback)    white
    ttk::setTheme strl
    option add *Text.background $anatempura::TEMPURA(tback) widgetDefault
    option add *highlightThickness 0
    option add *.font $anatempura::TEMPURA(HelvBig) widgetDefault
    ttk::style configure . -font $::anatempura::TEMPURA(HelvBig)
}


# tempura_spawn --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc tempura_spawn { } {
    set tmp [file join $anatempura::TEMPURA(tempura_path) $anatempura::TEMPURA(tempura_exe)]
    if {[file exists $tmp]} {
        set chan [open |[list $tmp 2>@1] a+]
        chan configure $chan -buffering line -blocking 0
        chan event $chan readable [list tempura_receive $chan]
        return $chan
    } else {
        tkterm::o_insert "Oh no, I can't find the Tempura executable.\n" sta
        return -1
    }
}

# tempura_receive --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc tempura_receive {tempura_io } {
    global reset show_output at_prompt at_input old_at_prompt syntax_error

    flush $tempura_io
    set result [chan gets $tempura_io] 
           
    switch -regexp -matchvar foo -- $result {
        "^Tempura \[0-9\]*%+ " {
            tkterm::o_insert "$result\n" tempu
            set show_output 1
            set at_prompt 1
            set at_input 0
        }
        "^!(Begin|End) state: \[0-9\]*" {
            tkterm::o_insert "$result\n" sta
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
        "^\[\x5b]\[^\x2e\]+\[\x5d]\[\x2e]" {
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
        "^(State\[^:]+: )?!TCL: (\[a-z_A-Z\]+) (\[^!\x01-\x1f\]+):" {
            if { [file exists $anatempura::TEMPURA(file_tcl)]} {
                if {[lindex $foo 2] == "init"} {
                    source $anatempura::TEMPURA(file_tcl)
                    visual::Window show .top23
                    set out::canv $anatempura::TEMPURA(out)
                    #set tkterm::stepper 0;
                    #set tkterm::run     0;
                    out::init [split [lindex $foo 3] ":"]
                } else {
                    if {[winfo exists .top23]} {
                        out::[lindex $foo 2] [split [lindex $foo 3] ":"]
                    }
                }
            }
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
        "^(State\[^:]+: )?!TCL: break :" {
            #tkterm::tbreak
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
        "^(State\[^:]+: )?!E: prog(\[0-9\]) (\[^!\x01-\x1f\]+):" {
            tkproggy::start [lindex $foo 2] [split [lindex $foo 3] ":"]
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
        "^(State\[^:]+: )?!E: veri(\[0-9\]) (\[\-0-9_a-zA-Z<=#\x20\]+):" {
            set tstop [string range [lindex $foo 3] 0 3]
            set ttime [string range [lindex $foo 3] 4 end]
            if {[lindex $foo 3] == "dot"} {
                tkproggy::startdot [lindex $foo 2]
            } elseif  {[lindex $foo 3] == "finish"} {
                tkproggy::startfinish [lindex $foo 2]
            } elseif  {$tstop == "stop"} {
                tkproggy::startstop [lindex $foo 2] $ttime
            } else {
                tkproggy::startveri [lindex $foo 2] [lindex $foo 3]
            }
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
        "^\\*\\*\\*Syntax Error" {
            tkterm::o_insert "$result\n" sta
            set syntax_error 1
            set show_output 1
            set at_prompt 0
            set at_input 0
        }
        "^((State\[^:]+: )|(\[ ]*))% Xin=\\?" {
            set show_output 0
            set at_prompt 0
            set at_input 1
        }
        "^((State\[^:]+: )|(\[ ]*))% (\[\-a-z_A-Z0-9\]|\[\x5b]|\[\x5d])+=\\?" {
            tkterm::o_insert "$result\n" tempu
            set show_output 1
            set at_prompt 0
            set at_input 1
        }
        "^(\[^\x01-\x1f]*)" {
            tkterm::o_insert "[lindex $foo 1]\n"
            set show_output 0
            set at_prompt 0
            set at_input 0
        }
    }
    if { [eof $tempura_io] } {
        chan close $tempura_io
        if {$reset == 0} {
            exit
        } else {
            set reset 0
        } 
    }
    if {$old_at_prompt != $at_prompt } {
        button_state
    }
    set old_at_prompt $at_prompt
    update idletasks   
}

# tempura_send --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc tempura_send {data } {
    global tempura_io show_output at_input
    if {$tempura_io != -1} {
        if {$show_output == 1} {
            tkterm::o_insert "$data\n" tempu
        }
        if [catch {chan puts $tempura_io $data} err] {
            tkterm::o_insert "Oh no, I can't contact Tempura anymore.\n" sta
        }
        flush $tempura_io
    } else {
        tkterm::o_insert "Oh no, I can't contact Tempura anymore.\n" sta
    }
}

# proggy_spawn --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc proggy_spawn { prog id} {
  set chan [open |$prog a+]
  chan configure $chan -buffering line -blocking 0
  chan event $chan readable [list proggy_receive $chan $id]
  return $chan

}

# proggy_receive --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc proggy_receive {proggy_io id} {
    global reset

    flush $proggy_io
    set result [chan gets $proggy_io]

    #puts "$id: receiving $result"
           
    switch -regexp -matchvar fooo -- $result {
      "^\x07" {
      }
      "^(State\[^:]+: )?!PROG: assert (\[^!\x01-\x1f\]+)!" {   
       tkproggy::tbreak $id [split [lindex $fooo 2] ":"]       
      }
      default { 
       tkterm::e_insert $id "$result\n"
      }
   }

    if {[lsearch [chan names] $proggy_io] !=-1} {
        if { [eof $proggy_io] } {
           chan close $proggy_io
           tkterm::e_insert $id "Done.\n" tempu
        }
    }

    update idletasks
}

# proggy_send --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc proggy_send {chan id data} {
    tkterm::e_insert $id "$data\n"
    chan puts $chan $data
    flush $chan
}

# watchfile --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc watchfile {} {
    if {$anatempura::TEMPURA(tempura_file) != "" && $anatempura::TEMPURA(tempura_dir)!=""} {
        set f [file join $anatempura::TEMPURA(tempura_dir) $anatempura::TEMPURA(tempura_file)]
        if { [file exists $f] } {
            set mtime [file mtime $f]
            if {$mtime != $anatempura::TEMPURA(mtime)} {
                .top18.cpd21.reloadButton configure -style Red.TButton
            } else {
                .top18.cpd21.reloadButton configure -style TButton
            }
        } else {
            tkterm::o_insert "Oh no, $f has disappeared.\n" sta
        }
    }
    set anatempura::TEMPURA(watch) [after 2500 watchfile]
}

# button_state --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc button_state { } {
    global at_prompt syntax_error

    if { $at_prompt } {
        .top18.cpd21.openButton configure -state normal
        if {$anatempura::TEMPURA(tempura_file) != "" && $anatempura::TEMPURA(tempura_dir)!=""} {
            .top18.cpd21.reloadButton configure -state normal
        }
        if {![string equal $::tcl_platform(platform) "windows"]} {
            .top18.cpd21.resetButton configure -state normal
        }
        .top18.cpd21.runButton configure -state normal
        .top18.cpd21.checkButton configure -state normal
        if { $anatempura::TEMPURA(rerun) != "" } {
            .top18.cpd21.rerunButton configure -state normal
        }
        .top18.cpd21.saveButton configure -state normal
        .top18.cpd21.clearButton configure -state normal
    } else {
        .top18.cpd21.openButton configure -state disabled
        .top18.cpd21.reloadButton configure -state disabled
        if {![string equal $::tcl_platform(platform) "windows"]} {
            .top18.cpd21.resetButton configure -state disabled
        }
        .top18.cpd21.runButton configure -state disabled
        .top18.cpd21.checkButton configure -state disabled
        .top18.cpd21.rerunButton configure -state disabled
        .top18.cpd21.saveButton configure -state disabled
        .top18.cpd21.clearButton configure -state disabled
    }

}

proc get_stdin {} {
    global tempura_io at_prompt proggy_iol
    set enter_text [chan gets stdin]
    set temp [string range $enter_text 0 3]

    if {$temp == "load"} {
        if {[string range $enter_text end end] == "." } {
            set temp [string trimright $enter_text "."]
        } else {
            set temp $enter_text
        }
        set tt [string range $temp 6 end-1]
        set anatempura::TEMPURA(base_file) [file normalize [file join [pwd] $tt.t]]   
        anatempura::newdir [file dir $anatempura::TEMPURA(base_file)]
        anatempura::setpath
        set anatempura::TEMPURA(tempura_file) [file tail $anatempura::TEMPURA(base_file)]
        if {[file exists $anatempura::TEMPURA(base_file)]} {
            anatempura::buildeditmenu [file join $anatempura::TEMPURA(tempura_dir) $anatempura::TEMPURA(tempura_file)]
        }     
        set ttt [file rootname $anatempura::TEMPURA(tempura_file)]
        tempura_send "load \"$ttt\"."
    } elseif {$temp == "run "} {
        if {[string range $enter_text end end] == "." } {
            set temp [string trimright $enter_text "."]
        } else {
            set temp $enter_text
        }
        if [catch {$anatempura::TEMPURA(runmenu) index $temp} index] {
            tempura_send "$enter_text"
            set anatempura::TEMPURA(rerun) "$enter_text"
            #set enter_text ""
        } else {
            $anatempura::TEMPURA(runmenu) invoke $index
            #set enter_text ""
        }
    } elseif {$temp == "exit"} {
        foreach id [array names proggy_iol] {
            anatempura::stopexternal $proggy_iol($id)
        }
        exit
    } else {
        if {$temp != ""} {
            tempura_send "$enter_text"
            #set enter_text ""
        }
    }
    
}


# main --
#
#   ADD COMMENTS HERE
#
# Arguments:
#   args	comments
# Results:
#   Returns ...
#
proc main {argc argv} {
    global tempura_io show_output proggy_iol veri enter_text old_at_prompt syntax_error reset argv0 env external externalwindow stdio

    set arglen [llength $argv]
    set index 0
    set stdio 0
    while {$index < $arglen} {
        set arg [lindex $argv $index]
        switch -exact -- $arg {
            {-stdio} { set stdio 1 }
            default  { set stdio 0 }
        }
        incr index
    }
    
    set tempdir [pwd]
    if {[info exists ::starkit::topdir]} {
        cd [file dirname $::starkit::topdir]
        set anatempura::TEMPURA(anatempura) $::starkit::topdir
    } else {
        cd [file dirname $argv0]
        set anatempura::TEMPURA(anatempura) [file normalize $argv0]
    }
    
    set anatempura::TEMPURA(step)             0
    set anatempura::TEMPURA(version)         "3.5"
    set anatempura::TEMPURA(base_file)       ""
    set anatempura::TEMPURA(tempura_file)    ""
    set anatempura::TEMPURA(file_tcl)        ""
    set anatempura::TEMPURA(rerun)           ""
    set anatempura::TEMPURA(file_lbl)        "<untitled>"
    set anatempura::TEMPURA(mtime)           0
    set anatempura::TEMPURA(watch)           0
    set tkterm::o_rows 10		
    set tkterm::e_rows 5				
    set tkterm::o_cols 80		
    set tkterm::e_cols 80		
    set tkterm::stepper 0
    set tkterm::run     0
    set veri 0
    set reset 0
    set show_output 1
    set at_prompt 0
    set old_at_prompt 0
    set at_input 0
    set syntax_error 1
    set enter_text ""
    set env(LINES) $tkterm::o_rows
    set env(COLUMNS) $tkterm::o_cols
    
    
    init $argc $argv
    anatempura::init

    if {$stdio == 0} {
        visual::Window show .
        visual::Window show .top18
        set tkterm::o_term $anatempura::TEMPURA(output)
    
        set tempura_io [tempura_spawn]
        set x 0
        after 500 {set x 1}
        vwait x
        anatempura::setpath
    } else {
        visual::Window show .
        visual::Window hide .
        visual::Window show .top18
        visual::Window hide .top18
        set tkterm::o_term $anatempura::TEMPURA(output)
        set tempura_io [tempura_spawn]
        cd $tempdir
        chan configure stdin -buffering line -blocking 0
        chan event stdin readable get_stdin
    }
}


main $argc $argv
