/* This file is part Tempura: Interval Temporal Logic interpreter.
 *  
 * Copyright (C) 1987-2019 Roger Hale, Ben Moszkowski, Antonio Cau 
 *
 * Tempura is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Tempura is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Changes made by  AC: Antonio Cau
 *                 BCM: Ben Moszkowski
 *
 * $Id: r-io.c 284 2017-10-08 08:00:31Z cau $
 */


#include "system.h"




/***************************\
*                           *
*  input-output statements  *
*                           *
\***************************/

/* WINPUT e */
void reduce_WINPUT(e)
NODEPTR e;
{
    NODEPTR val,tval;
    NODEPTR loc = reduce_ref (node_lnk_1(e));
    if (Ready) {
        output_state_number;
        fputs("% ", stdout); /* AC: added ? */
	output_expn (node_lnk_1(e), -1);
        fputs ("=?\n", stdout); /* AC: added ? */
        fflush(stdout);
        if (strcmp (node_strval(InFile), "stdin") == 0) {
            INTERACTIVE = 1;
            tval = parse(stdin);
            if (yyerrorcount == 0) {
                val = reduce_val (tval);
            } else {
                error_break0("error in winput");
            }
        }
        else {
            if (Fpcur == NULL) {
                warn1 ("cannot open file \"%s\"",  node_strval(InFile));
                INTERACTIVE = 1;
                tval = parse(stdin);
                if (yyerrorcount == 0) {
                    val = reduce_val (tval);
                } else {
                    error_break0("error in winput");
                }
            }
            else{
                INTERACTIVE = 1;
                val = parse (Fpcur);
                while ( EOFILE == 1)  {
                    EOFILE=0;
                    val = parse (Fpcur);
                }
                INTERACTIVE = 1;
                if (yyerrorcount == 0) {
                    val = reduce_val (val);
                } else {
                    error_break0("error in winput");
                }
            }
        }
        
        
        
        if (Ready) {
	    if (!AssignAheadFlag && is_next (loc)) {
	        result (assign (node_lnk_1(e), val));
            }
	    else {
		assign_val1 (loc, val);
	        result (True);
	    }
	}
    }
}


/* INPUT e */
void reduce_INPUT(e)
NODEPTR e;
{
    NODEPTR val,tval;
    NODEPTR loc = reduce_ref (node_lnk_1(e));
    if (Ready) {
        output_state_number;
        foutput_str("% "); /* AC: added ? */
	foutput_expn (node_lnk_1(e), -1);
        foutput_str("=?"); /* AC: added ? */
        fnewline;
        fflush(stdout);
        if (strcmp (node_strval(InFile), "stdin") == 0) {
            INTERACTIVE = 1;
            tval = parse(stdin);
            if (yyerrorcount == 0) {
                val = reduce_val (tval);
            } else {
                error_break0("error in input");
            }
        }
        else {
            if (Fpcur == NULL) {
                warn1 ("cannot open file \"%s\"",  node_strval(InFile));
                INTERACTIVE = 1;
                tval = parse(stdin);
                if (yyerrorcount == 0) {
                    val = reduce_val (tval);
                } else {
                   error_break0("error in input");
                }
            }
            else {
                INTERACTIVE = 1;
                val = parse (Fpcur);
                if (yyerrorcount == 0) {
                    if (EOFILE == 1) {
                        warn0 ("end of file will use stdin");
                        fclose (Fpcur);
                        INTERACTIVE = 1;
                        tval = parse(stdin);
                        if (yyerrorcount == 0) {
                            val = reduce_val (tval);
                        } else {
                            error_break0("error in input");
                        }
                    } else {
                        val = reduce_val (val);
                    }
                } else {
                    error_break0("error in input");
                }
            }
        }
        
        if (Ready) {
	    if (!AssignAheadFlag && is_next (loc)) {
	        result (assign (node_lnk_1(e), val));
            }
	    else {
		assign_val1 (loc, val);
	        result (True);
	    }
	}
    }
}

/* OUTPUT e1 */
void reduce_OUTPUT(e)
NODEPTR e;
{
    NODEPTR e1 = reduce_ref_and_fetch1 (node_lnk_1(e));
    unsigned und = reduce_ref_and_fetch_is_undefined(node_lnk_1(e));
    
        /*unsigned hasd = reduce_ref_and_fetch_has_default(node_lnk_1(e));*/
    
        /*if (DebugLevel>2)
        if (node_type(node_lnk_1(e)) == STATE || node_type(node_lnk_1(e)) == STATIC)
        printf("reduce_OUTPUT: history=%d\n",history(node_lnk_1(e)));*/
    
    if ((Ready && val_defined (e1)) || (ForceOutput && und) || (!StateProgressFlag)) {
        output_state_number;
        if (strcmp (node_strval(OutFile), "stdout") != 0) {
            newline;
        }
        if (DebugLevel>=5) {
            printf("IU=%1d: ",und);
         }
        foutput_expn (node_lnk_1(e), -1);
        foutput_str ("=");
        if (Ready)
            foutput_val (e1);
        else
            foutput_char ('?');
        fnewline;
        result (True);
    }
    Ready = (Ready && val_defined (e1)) || (ForceOutput && und) || (!StateProgressFlag);
}

/* iREMEMBER e1 */
void reduce_iREMEMBER(e)
NODEPTR e;
{
    NODEPTR e1 = reduce_ref_and_fetch1 (node_lnk_1(e));
        /*unsigned und = reduce_ref_and_fetch_is_undefined(node_lnk_1(e));*/
    
        /*unsigned hasd = reduce_ref_and_fetch_has_default(node_lnk_1(e));*/
    
        /*if (DebugLevel>2)
        if (node_type(node_lnk_1(e)) == STATE || node_type(node_lnk_1(e)) == STATIC)
        printf("reduce_OUTPUT: history=%d\n",history(node_lnk_1(e)));*/
    if (DebugLevel>=5) {
            printf("***reduce_iREMEMBER: Ready=%d VD=%d FO=%d PPF=%d SPF=%d\n",Ready,val_defined2(e1),ForceOutput,PassProgressFlag,StateProgressFlag);
    }
    
    if ((Ready && val_defined (e1)) || (!StateProgressFlag) ) {
        result (True);
    }
    Ready = ((Ready && val_defined (e1)) || (!StateProgressFlag));
}


/* PEEK e */
void reduce_PEEK(e)
NODEPTR e;
{
    NODEPTR e1;
    foutput_expn (node_lnk_1(e), -1);
    foutput_str ("=");
    e1 = reduce_ref_and_fetch1 (node_lnk_1(e));
    if (Ready)
        foutput_val (e1);
    else
        foutput_char ('?');
    fnewline;
    Ready = T;
    result (True);
}

/* Output a value according to a given print format. */
/* FORMAT elist */
void reduce_FORMAT(e)
NODEPTR e;
{
    NODEPTR elist = node_lnk_1(e);
    NODEPTR fmt;
    NODEPTR item;
    NODEPTR base_loc,
            loc;
    unsigned und;
    int     field_size;
    int     na=0;
    int     nf=0;
    char   *fmt_ptr;
    char   name[MAXFILENAMESIZE];
    FILE * fp = NULL,
         *openfile ();

    assert0 (elist != NULL, "format statement must contain format string");
    loc = base_loc = node_blk_base(mem_block2 (STATE, elist_size (elist),1,NoFraming));
    Ready = T; und = T;
    while ((Ready || (ForceOutput && und) || (!StateProgressFlag)) && (elist != NULL)) {
        item = reduce_ref_and_fetch1 (node_lnk_1(elist));
        na=na+1;
        und = und && reduce_ref_and_fetch_is_undefined(node_lnk_1(elist));
        if (Ready) {
            if ((Ready=val_defined (item))) {
                put_mem1 (curr_loc (loc), item);
            }
	}
        elist = node_lnk_2(elist);
        loc += offset2 (1,node_mem_history(loc));
    }
    if (!StateProgressFlag)
        Ready=T;
    if (Ready || (ForceOutput && und) ) {
        type_check (fmt = get_mem (loc = curr_loc (base_loc)), STRING);
        na=na-1;
        for (fmt_ptr = node_strval(fmt); *fmt_ptr != '\0'; fmt_ptr += 1) {
            if (*fmt_ptr == '%') { nf =nf+1;
            }
        }
        assert0 (na == nf, "number of arguments does not match");
        fmt = get_mem (loc = curr_loc (base_loc));
        output_state_number;
        for (fmt_ptr = node_strval(fmt); *fmt_ptr != '\0'; fmt_ptr += 1) {
            if (*fmt_ptr != '%') {
                if (strcmp (node_strval(OutFile), "stdout") == 0) {
                    putchar (*fmt_ptr);
                }
                else {
                    if ((fp = openfile (name, node_strval(Path), node_strval(OutFile), FXOUT, "a")) == NULL) {
                        warn1 ("cannot open file \"%s\"",  node_strval(OutFile));
                        warn0 ("will display to stdout");
                        putchar (*fmt_ptr);
                    }
                    else {
                        putc(*fmt_ptr,fp);
                        fclose (fp);
                    }
                }
            }
            else {
                CharsOutput = 0;
                field_size = 0;
                while (isdigit ((int)*++fmt_ptr)) {
                    field_size = field_size * 10 + *fmt_ptr - '0';
                }
                switch (*fmt_ptr) {
                    case 'd':
                    case 'D':
		        if (mem_defined1 (loc += offset2 (1,node_mem_history(loc)))) {
			    type_check (item = get_mem (loc), INT);
			    foutput_int (node_ival(item), field_size);
			} else 
			    foutputf_str ("?", field_size);
                        break;
                    case 's':
                    case 'S':
		        if (mem_defined1 (loc += offset2 (1,node_mem_history(loc)))) {
                            type_check (item = get_mem (loc), STRING);
                            foutputf_str (node_strval(item), field_size);
			} else 
			    foutputf_str ("?", field_size);
                        break;
                    case 'e':
                    case 'E':
		        if (mem_defined1 (loc += offset2 (1,node_mem_history(loc)))) {
                            type_check (item = get_mem (loc), FLOAT);
                            foutputf_fl (node_flval(item), field_size);
			} else 
			    foutputf_str ("?", field_size);
                        break;
                    case 'f':
                    case 'F':
		        if (mem_defined1 (loc += offset2 (1,node_mem_history(loc)))) {
                            type_check (item = get_mem (loc), FLOAT);
                            foutputf_fl_f (node_flval(item), field_size);
			} else 
			    foutputf_str ("?", field_size);
                        break;    
                    case 't':
                    case 'T':
		        if (mem_defined1 (loc += offset2 (1,node_mem_history(loc)))) {
                            foutput_expn (item = get_mem (loc), -1);
                        }
			else {
			    foutputf_str ("?", field_size);
                        }
                        break;
                    case '#':
                        foutput_int ((int)StateNumber, field_size);
                        break;
                    default:
                        if (strcmp (node_strval(OutFile), "stdout") == 0) {
                            putchar (*fmt_ptr);
                        }
                        else {
                            if ((fp = openfile (name, node_strval(Path), node_strval(OutFile), FXOUT, "a")) == NULL) {
                                warn1 ("cannot open file \"%s\"",  node_strval(OutFile));
                                warn0 ("will display to stdout");
                                putchar (*fmt_ptr);
                            }
                            else {
                                putc(*fmt_ptr,fp);
                                fclose (fp);
                            }
                        }
                }
                fspaces (field_size - CharsOutput);
            }
        }
        result (True);
    }
}
